@@ -19,18 +19,18 @@
 
 package org.apache.sysds.runtime.compress.estim;
 
-import java.util.Arrays;
 import java.util.EnumMap;
 import java.util.Map;
 import java.util.Set;
 
-import org.apache.commons.lang.NotImplementedException;
+import org.apache.commons.lang3.NotImplementedException;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.apache.sysds.runtime.compress.CompressionSettings;
 import org.apache.sysds.runtime.compress.DMLCompressionException;
 import org.apache.sysds.runtime.compress.colgroup.AColGroup.CompressionType;
 import org.apache.sysds.runtime.compress.colgroup.ColGroupSizes;
+import org.apache.sysds.runtime.compress.colgroup.indexes.IColIndex;
 import org.apache.sysds.runtime.compress.estim.encoding.IEncode;
 
 /**
@@ -40,62 +40,113 @@ public class CompressedSizeInfoColGroup {
 
 	protected static final Log LOG = LogFactory.getLog(CompressedSizeInfoColGroup.class.getName());
 
-	private final int[] _cols;
+	private final IColIndex _cols;
 	private final EstimationFactors _facts;
-	private final double _cardinalityRatio;
-	private final long _minSize;
+	private final double _minSize;
 	private final CompressionType _bestCompressionType;
-	private final EnumMap<CompressionType, Long> _sizes;
+	private final EnumMap<CompressionType, Double> _sizes;
 
 	/**
 	 * Map containing a mapping to unique values, but not necessarily the actual values contained in this column group
 	 */
 	private IEncode _map;
 
-	protected CompressedSizeInfoColGroup(int[] columns, EstimationFactors facts,
+	public CompressedSizeInfoColGroup(IColIndex cols, int nVal, int nRow, CompressionType bestCompressionType) {
+		_cols = cols;
+		_facts = new EstimationFactors(nVal, nRow);
+		_minSize = -1;
+		_bestCompressionType = bestCompressionType;
+		_sizes = new EnumMap<>(CompressionType.class);
+		_sizes.put(bestCompressionType, _minSize);
+	}
+
+	public CompressedSizeInfoColGroup(IColIndex cols, EstimationFactors facts, CompressionType bestCompressionType) {
+		_cols = cols;
+		_facts = facts;
+		_minSize = -1;
+		_bestCompressionType = bestCompressionType;
+		_sizes = new EnumMap<>(CompressionType.class);
+		_sizes.put(bestCompressionType, _minSize);
+	}
+
+	public CompressedSizeInfoColGroup(IColIndex cols, EstimationFactors facts, long minSize,
+		CompressionType bestCompressionType) {
+		_cols = cols;
+		_facts = facts;
+		_minSize = minSize;
+		_bestCompressionType = bestCompressionType;
+		_sizes = new EnumMap<>(CompressionType.class);
+		_sizes.put(bestCompressionType, _minSize);
+	}
+
+	public CompressedSizeInfoColGroup(IColIndex columns, EstimationFactors facts, long minSize,
+		CompressionType bestCompression, IEncode map) {
+		_cols = columns;
+		_facts = facts;
+		_minSize = minSize;
+		_bestCompressionType = bestCompression;
+		_sizes = new EnumMap<>(CompressionType.class);
+		_sizes.put(bestCompression, _minSize);
+		_map = map;
+	}
+
+	public CompressedSizeInfoColGroup(IColIndex columns, EstimationFactors facts,
 		Set<CompressionType> validCompressionTypes, IEncode map) {
 		_cols = columns;
 		_facts = facts;
-		_cardinalityRatio = (double) facts.numVals / facts.numRows;
-		_sizes = calculateCompressionSizes(_cols.length, facts, validCompressionTypes);
-		Map.Entry<CompressionType, Long> bestEntry = null;
-		for(Map.Entry<CompressionType, Long> ent : _sizes.entrySet()) {
-			if(bestEntry == null || ent.getValue() < bestEntry.getValue())
-				bestEntry = ent;
+		_sizes = calculateCompressionSizes(_cols, facts, validCompressionTypes);
+
+		CompressionType tmpBestCompressionType = CompressionType.UNCOMPRESSED;
+		double tmpBestCompressionSize = _sizes.getOrDefault(tmpBestCompressionType, Double.MAX_VALUE);
+		for(Map.Entry<CompressionType, Double> ent : _sizes.entrySet()) {
+			if(ent.getValue() < tmpBestCompressionSize) {
+				tmpBestCompressionType = ent.getKey();
+				tmpBestCompressionSize = ent.getValue();
+			}
 		}
 
-		_bestCompressionType = bestEntry.getKey();
-		_minSize = bestEntry.getValue();
+		_bestCompressionType = tmpBestCompressionType;
+		_minSize = tmpBestCompressionSize;
 		_map = map;
-		if(LOG.isTraceEnabled())
-			LOG.trace(this);
 	}
 
 	/**
-	 * Create empty.
+	 * Create empty or const.
 	 * 
 	 * @param columns columns
 	 * @param nRows   number of rows
+	 * @param ct      The type intended either Empty or Const
 	 */
-	public CompressedSizeInfoColGroup(int[] columns, int nRows) {
+	public CompressedSizeInfoColGroup(IColIndex columns, int nRows, CompressionType ct) {
 		_cols = columns;
-		_facts = new EstimationFactors(columns.length, 0, nRows);
-		_cardinalityRatio = 0;
+		_facts = new EstimationFactors(0, nRows);
 		_sizes = new EnumMap<>(CompressionType.class);
-		final CompressionType ct = CompressionType.EMPTY;
-		_sizes.put(ct,  ColGroupSizes.estimateInMemorySizeEMPTY(columns.length));
+		switch(ct) {
+			case EMPTY:
+				_sizes.put(ct, (double) ColGroupSizes.estimateInMemorySizeEMPTY(columns.size(), columns.isContiguous()));
+				break;
+			case CONST:
+				_sizes.put(ct,
+					(double) ColGroupSizes.estimateInMemorySizeCONST(columns.size(), columns.isContiguous(), 1.0, false));
+				break;
+			case UNCOMPRESSED:
+				_sizes.put(ct, (double) ColGroupSizes.estimateInMemorySizeUncompressed(nRows, columns.isContiguous(),
+					columns.size(), 1.0));
+				break;
+			default:
+				throw new DMLCompressionException("Invalid instantiation of const Cost");
+		}
 		_bestCompressionType = ct;
 		_minSize = _sizes.get(ct);
 		_map = null;
-
 	}
 
-	public long getCompressionSize(CompressionType ct) {
+	public double getCompressionSize(CompressionType ct) {
 		if(_sizes != null) {
-			Long s = _sizes.get(ct);
+			Double s = _sizes.get(ct);
 			if(s == null)
 				throw new DMLCompressionException("Asked for valid " + ct + " but got null. contains:" + _sizes);
-			return _sizes.get(ct);
+			return s;
 		}
 		else
 			throw new DMLCompressionException("There was no encodings analyzed");
@@ -109,11 +160,11 @@ public class CompressedSizeInfoColGroup {
 		return _bestCompressionType;
 	}
 
-	public Map<CompressionType, Long> getAllCompressionSizes() {
+	public Map<CompressionType, Double> getAllCompressionSizes() {
 		return _sizes;
 	}
 
-	public long getMinSize() {
+	public double getMinSize() {
 		return _minSize;
 	}
 
@@ -135,7 +186,7 @@ public class CompressedSizeInfoColGroup {
 		return _facts.numOffs;
 	}
 
-	public int[] getColumns() {
+	public IColIndex getColumns() {
 		return _cols;
 	}
 
@@ -143,10 +194,6 @@ public class CompressedSizeInfoColGroup {
 		return _facts.numRows;
 	}
 
-	public double getCardinalityRatio() {
-		return _cardinalityRatio;
-	}
-
 	public double getMostCommonFraction() {
 		return (double) _facts.largestOff / _facts.numRows;
 	}
@@ -163,15 +210,21 @@ public class CompressedSizeInfoColGroup {
 		return _map;
 	}
 
+	public void setMap(IEncode map) {
+		_map = map;
+	}
+
 	public boolean containsZeros() {
 		return _facts.numOffs < _facts.numRows;
 	}
 
-	private static EnumMap<CompressionType, Long> calculateCompressionSizes(int numCols, EstimationFactors fact,
+	private static EnumMap<CompressionType, Double> calculateCompressionSizes(IColIndex cols, EstimationFactors fact,
 		Set<CompressionType> validCompressionTypes) {
-		EnumMap<CompressionType, Long> res = new EnumMap<CompressionType, Long>(CompressionType.class);
+		if(validCompressionTypes.size() > 10 )
+			throw new DMLCompressionException("Invalid big number of compression types");
+		EnumMap<CompressionType, Double> res = new EnumMap<>(CompressionType.class);
 		for(CompressionType ct : validCompressionTypes) {
-			long compSize = getCompressionSize(numCols, ct, fact);
+			double compSize = getCompressionSize(cols, ct, fact);
 			if(compSize > 0)
 				res.put(ct, compSize);
 		}
@@ -179,43 +232,52 @@ public class CompressedSizeInfoColGroup {
 	}
 
 	public boolean isEmpty() {
-		return _bestCompressionType == CompressionType.EMPTY;
+		return _bestCompressionType == CompressionType.EMPTY || _sizes.containsKey(CompressionType.EMPTY);
 	}
 
 	public boolean isConst() {
-		return _bestCompressionType == CompressionType.CONST;
+		return _bestCompressionType == CompressionType.CONST || _sizes.containsKey(CompressionType.CONST);
 	}
 
-	private static long getCompressionSize(int numCols, CompressionType ct, EstimationFactors fact) {
+	public boolean isIncompressable() {
+		return _bestCompressionType == CompressionType.UNCOMPRESSED;
+	}
+
+	private static double getCompressionSize(IColIndex cols, CompressionType ct, EstimationFactors fact) {
 		int nv;
+		final int numCols = cols.size();
+		final boolean contiguousColumns = cols.isContiguous();
 		switch(ct) {
-			case DeltaDDC: // TODO add proper extraction
+			case LinearFunctional:
+				return ColGroupSizes.estimateInMemorySizeLinearFunctional(numCols, contiguousColumns);
+			case DeltaDDC:
+				throw new NotImplementedException();
 			case DDC:
 				nv = fact.numVals + (fact.numOffs < fact.numRows ? 1 : 0);
-				// + 1 if the column contains zero
-				return ColGroupSizes.estimateInMemorySizeDDC(numCols, nv, fact.numRows, fact.tupleSparsity, fact.lossy);
+				return ColGroupSizes.estimateInMemorySizeDDC(numCols, contiguousColumns, nv, fact.numRows,
+					fact.tupleSparsity, fact.lossy);
 			case RLE:
-				throw new NotImplementedException();
-			// nv = fact.numVals + (fact.zeroIsMostFrequent ? 1 : 0);
-			// return ColGroupSizes.estimateInMemorySizeRLE(numCols, nv, fact.numRuns, fact.numRows, fact.tupleSparsity,
-			// fact.lossy);
+				return ColGroupSizes.estimateInMemorySizeRLE(numCols, contiguousColumns, fact.numVals, fact.numRuns,
+					fact.numRows, fact.tupleSparsity, fact.lossy);
 			case OLE:
 				nv = fact.numVals + (fact.zeroIsMostFrequent ? 1 : 0);
-				return ColGroupSizes.estimateInMemorySizeOLE(numCols, nv, fact.numOffs + fact.numVals, fact.numRows,
-					fact.tupleSparsity, fact.lossy);
+				return ColGroupSizes.estimateInMemorySizeOLE(numCols, contiguousColumns, nv, fact.numOffs + fact.numVals,
+					fact.numRows, fact.tupleSparsity, fact.lossy);
 			case UNCOMPRESSED:
-				return ColGroupSizes.estimateInMemorySizeUncompressed(fact.numRows, numCols, fact.overAllSparsity);
+				return ColGroupSizes.estimateInMemorySizeUncompressed(fact.numRows, contiguousColumns, numCols,
+					fact.overAllSparsity);
 			case SDC:
-				return ColGroupSizes.estimateInMemorySizeSDC(numCols, fact.numVals, fact.numRows, fact.largestOff,
-					fact.tupleSparsity, fact.zeroIsMostFrequent, fact.lossy);
+				return ColGroupSizes.estimateInMemorySizeSDC(numCols, contiguousColumns, fact.numVals, fact.numRows,
+					fact.largestOff, fact.tupleSparsity, fact.zeroIsMostFrequent, fact.lossy);
 			case CONST:
 				if(fact.numOffs == fact.numRows && fact.numVals == 1)
-					return ColGroupSizes.estimateInMemorySizeCONST(numCols, fact.tupleSparsity, fact.lossy);
+					return ColGroupSizes.estimateInMemorySizeCONST(numCols, contiguousColumns, fact.tupleSparsity,
+						fact.lossy);
 				else
 					return -1;
 			case EMPTY:
 				if(fact.numOffs == 0)
-					return ColGroupSizes.estimateInMemorySizeEMPTY(numCols);
+					return ColGroupSizes.estimateInMemorySizeEMPTY(numCols, contiguousColumns);
 				else
 					return -1;
 			default:
@@ -231,12 +293,20 @@ public class CompressedSizeInfoColGroup {
 	public String toString() {
 		StringBuilder sb = new StringBuilder();
 		sb.append(this.getClass().getSimpleName());
-		sb.append("cols: " + Arrays.toString(_cols));
+		sb.append(" cols: " + _cols);
 		sb.append(String.format(" common: %4.3f", getMostCommonFraction()));
-		sb.append(" Sizes: ");
-		sb.append(_sizes);
+		sb.append(" Sizes: " + _sizes);
 		sb.append(" facts: " + _facts);
-		sb.append("\n" + _map);
+		sb.append(" mapIsNull: " + (_map == null));
+		if(_map != null) {
+			String s = _map.toString();
+			if(s.length() > 1000) {
+				sb.append(s, 0, 1000);
+			}
+			else {
+				sb.append(s);
+			}
+		}
 		return sb.toString();
 	}
 
