@@ -19,9 +19,26 @@
 
 package org.apache.sysds.runtime.controlprogram;
 
+import java.io.Serializable;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collection;
+import java.util.Collections;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.List;
+import java.util.Set;
+import java.util.stream.Collectors;
+import java.util.stream.IntStream;
+import java.util.stream.Stream;
+import java.util.Queue;
+
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
 import org.apache.log4j.Level;
 import org.apache.sysds.api.DMLScript;
 import org.apache.sysds.common.Types.DataType;
+import org.apache.sysds.common.Types.ExecType;
 import org.apache.sysds.common.Types.FileFormat;
 import org.apache.sysds.common.Types.ValueType;
 import org.apache.sysds.conf.CompilerConfig;
@@ -29,7 +46,6 @@ import org.apache.sysds.conf.ConfigurationManager;
 import org.apache.sysds.hops.OptimizerUtils;
 import org.apache.sysds.hops.recompile.Recompiler;
 import org.apache.sysds.lops.Lop;
-import org.apache.sysds.common.Types.ExecType;
 import org.apache.sysds.parser.DMLProgram;
 import org.apache.sysds.parser.DataIdentifier;
 import org.apache.sysds.parser.ParForStatementBlock;
@@ -59,20 +75,11 @@ import org.apache.sysds.runtime.controlprogram.parfor.ResultMergeLocalMemory;
 import org.apache.sysds.runtime.controlprogram.parfor.ResultMergeRemoteSpark;
 import org.apache.sysds.runtime.controlprogram.parfor.Task;
 import org.apache.sysds.runtime.controlprogram.parfor.TaskPartitioner;
-import org.apache.sysds.runtime.controlprogram.parfor.TaskPartitionerFactoring;
-import org.apache.sysds.runtime.controlprogram.parfor.TaskPartitionerFactoringCmax;
-import org.apache.sysds.runtime.controlprogram.parfor.TaskPartitionerFactoringCmin;
-import org.apache.sysds.runtime.controlprogram.parfor.TaskPartitionerFixedsize;
-import org.apache.sysds.runtime.controlprogram.parfor.TaskPartitionerNaive;
-import org.apache.sysds.runtime.controlprogram.parfor.TaskPartitionerStatic;
+import org.apache.sysds.runtime.controlprogram.parfor.TaskPartitionerFactory;
 import org.apache.sysds.runtime.controlprogram.parfor.opt.OptTreeConverter;
 import org.apache.sysds.runtime.controlprogram.parfor.opt.OptimizationWrapper;
 import org.apache.sysds.runtime.controlprogram.parfor.opt.OptimizerRuleBased;
 import org.apache.sysds.runtime.controlprogram.parfor.opt.ProgramRecompiler;
-import org.apache.sysds.runtime.controlprogram.parfor.stat.InfrastructureAnalyzer;
-import org.apache.sysds.runtime.controlprogram.parfor.stat.Stat;
-import org.apache.sysds.runtime.controlprogram.parfor.stat.StatisticMonitor;
-import org.apache.sysds.runtime.controlprogram.parfor.stat.Timing;
 import org.apache.sysds.runtime.controlprogram.parfor.util.IDHandler;
 import org.apache.sysds.runtime.controlprogram.parfor.util.IDSequence;
 import org.apache.sysds.runtime.instructions.cp.BooleanObject;
@@ -84,26 +91,17 @@ import org.apache.sysds.runtime.instructions.cp.ScalarObject;
 import org.apache.sysds.runtime.instructions.cp.StringObject;
 import org.apache.sysds.runtime.instructions.cp.VariableCPInstruction;
 import org.apache.sysds.runtime.lineage.Lineage;
+import org.apache.sysds.runtime.lineage.LineageCacheConfig;
 import org.apache.sysds.runtime.lineage.LineageItem;
 import org.apache.sysds.runtime.lineage.LineageItemUtils;
 import org.apache.sysds.runtime.meta.DataCharacteristics;
 import org.apache.sysds.runtime.util.CollectionUtils;
+import org.apache.sysds.runtime.util.CommonThreadPool;
 import org.apache.sysds.runtime.util.ProgramConverter;
 import org.apache.sysds.runtime.util.UtilFunctions;
+import org.apache.sysds.utils.stats.InfrastructureAnalyzer;
 import org.apache.sysds.utils.stats.ParForStatistics;
-
-import java.io.Serializable;
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.Collection;
-import java.util.Collections;
-import java.util.HashMap;
-import java.util.HashSet;
-import java.util.List;
-import java.util.Set;
-import java.util.stream.Collectors;
-import java.util.stream.IntStream;
-import java.util.stream.Stream;
+import org.apache.sysds.utils.stats.Timing;
 
 
 
@@ -117,8 +115,8 @@ import java.util.stream.Stream;
  * TODO: papply(A,1:2,FUN) language construct (compiled to ParFOR) via DML function repository =&gt; modules OK, but second-order functions required
  *
  */
-public class ParForProgramBlock extends ForProgramBlock 
-{	
+public class ParForProgramBlock extends ForProgramBlock {	
+	protected static final Log LOG = LogFactory.getLog(ParForProgramBlock.class.getName());
 	// execution modes
 	public enum PExecMode {
 		LOCAL,          //local (master) multi-core execution mode
@@ -317,7 +315,6 @@ public class ParForProgramBlock extends ForProgramBlock
 	
 	// runtime parameters
 	protected final HashMap<String,String> _params;
-	protected final boolean _monitor;
 	protected final Level _optLogLevel;
 	protected int _numThreads = -1;
 	protected boolean _fixedDOP = false; //guard for numThreads
@@ -358,7 +355,7 @@ public class ParForProgramBlock extends ForProgramBlock
 	
 	protected long _ID = -1;
 	protected int _IDPrefix = -1;
-	protected boolean _monitorReport = false;
+	protected int _numRuns = -1;
 	
 	// local parworker data
 	protected HashMap<Long,ArrayList<ProgramBlock>> _pbcache = null;
@@ -399,7 +396,6 @@ public class ParForProgramBlock extends ForProgramBlock
 			_execMode        = PExecMode.valueOf( getParForParam(ParForStatementBlock.EXEC_MODE) );
 			_optMode         = POptMode.valueOf( getParForParam(ParForStatementBlock.OPT_MODE) );
 			_optLogLevel     = Level.toLevel( getParForParam(ParForStatementBlock.OPT_LOG) );
-			_monitor         = (Integer.parseInt(getParForParam(ParForStatementBlock.PROFILE) ) == 1);
 		}
 		catch(Exception ex) {
 			throw new RuntimeException("Error parsing specified ParFOR parameters.",ex);
@@ -421,7 +417,7 @@ public class ParForProgramBlock extends ForProgramBlock
 			_pbcache = new HashMap<>();
 		
 		//created profiling report after parfor exec
-		_monitorReport = _monitor;
+		_numRuns = 0;
 		
 		//materialized meta data (reused for all invocations)
 		_hasFunctions = ProgramRecompiler.containsAtLeastOneFunction(this);
@@ -537,10 +533,6 @@ public class ParForProgramBlock extends ForProgramBlock
 		_jvmReuse = false;
 	}
 	
-	public void disableMonitorReport() {
-		_monitorReport = false;
-	}
-	
 	public void setResultMerge(PResultMerge merge) {
 		_resultMerge = merge;
 		_params.put(ParForStatementBlock.RESULT_MERGE, String.valueOf(_resultMerge)); //kept up-to-date for copies
@@ -574,7 +566,7 @@ public class ParForProgramBlock extends ForProgramBlock
 	public void execute(ExecutionContext ec)
 	{
 		ParForStatementBlock sb = (ParForStatementBlock)getStatementBlock();
-
+		
 		// evaluate from, to, incr only once (assumption: known at for entry)
 		ScalarObject from0 = executePredicateInstructions(1, _fromInstructions, ec, false);
 		ScalarObject to0   = executePredicateInstructions(2, _toInstructions, ec, false);
@@ -589,8 +581,15 @@ public class ParForProgramBlock extends ForProgramBlock
 		//early exit on num iterations (e.g., for invalid loop bounds)
 		_numIterations = UtilFunctions.getSeqLength( 
 			from0.getDoubleValue(), to0.getDoubleValue(), incr0.getDoubleValue(), false);
+		// avoid unnecessary optimization/initialization, and issue with simplification rewrites
+		// (e.g., rewriting leftindexing into a result variable, to assignment)
 		if( _numIterations <= 0 )
-			return; //avoid unnecessary optimization/initialization
+			return; 
+		if( _numIterations == 1 ) {
+			//fallback to basic for loop.
+			super.execute(ec);
+			return;
+		}
 		
 		IntObject from = new IntObject(from0.getLongValue());
 		IntObject to = new IntObject(to0.getLongValue());
@@ -600,14 +599,12 @@ public class ParForProgramBlock extends ForProgramBlock
 		//OPTIMIZATION of ParFOR body (incl all child parfor PBs)
 		///////
 		if( _optMode != POptMode.NONE ) {
-			OptimizationWrapper.setLogLevel(_optLogLevel); //set optimizer log level
-			OptimizationWrapper.optimize(_optMode, sb, this, ec, _monitor); //core optimize
+			OptimizationWrapper.optimize(_optMode, sb, this, ec, _numRuns); //core optimize
 		}
 
 		///////
 		//DATA PARTITIONING of read-only parent variables of type (matrix,unpartitioned)
 		///////
-		Timing time = _monitor ? new Timing(true) : null;
 		
 		//partitioning on demand (note: for fused data partitioning and execute the optimizer set 
 		//the data partitioner to NONE in order to prevent any side effects)
@@ -619,10 +616,7 @@ public class ParForProgramBlock extends ForProgramBlock
 		//eager rdd caching of variables for spark in order prevent read/write contention
 		handleSparkEagerCaching( ec );
 		
-		if( _monitor ) 
-			StatisticMonitor.putPFStat(_ID, Stat.PARFOR_INIT_DATA_T, time.stop());
-		
-		// initialize iter var to form value
+		// initialize iter var to from value
 		IntObject iterVar = new IntObject(from.getLongValue());
 		
 		///////
@@ -630,17 +624,9 @@ public class ParForProgramBlock extends ForProgramBlock
 		///////
 		LOG.trace("EXECUTE PARFOR ID = "+_ID+" with mode = "+_execMode+", numThreads = "+_numThreads+", taskpartitioner = "+_taskPartitioner);
 		
-		if( _monitor ) {
-			StatisticMonitor.putPFStat(_ID, Stat.PARFOR_NUMTHREADS,      _numThreads);
-			StatisticMonitor.putPFStat(_ID, Stat.PARFOR_TASKSIZE,        _taskSize);
-			StatisticMonitor.putPFStat(_ID, Stat.PARFOR_TASKPARTITIONER, _taskPartitioner.ordinal());
-			StatisticMonitor.putPFStat(_ID, Stat.PARFOR_DATAPARTITIONER, _dataPartitioner.ordinal());
-			StatisticMonitor.putPFStat(_ID, Stat.PARFOR_EXECMODE,        _execMode.ordinal());
-		}
-		
 		//preserve shared input/result variables of cleanup
 		ArrayList<String> varList = ec.getVarList();
-		boolean[] varState = ec.pinVariables(varList);
+		Queue<Boolean> varState = ec.pinVariables(varList);
 		
 		try 
 		{
@@ -665,7 +651,7 @@ public class ParForProgramBlock extends ForProgramBlock
 		catch(Exception ex) {
 			throw new DMLRuntimeException("PARFOR: Failed to execute loop in parallel.",ex);
 		}
-		
+
 		//reset state of shared input/result variables 
 		ec.unpinVariables(varList, varState);
 		
@@ -695,8 +681,7 @@ public class ParForProgramBlock extends ForProgramBlock
 		///////
 	
 		//print profiling report (only if top-level parfor because otherwise in parallel context)
-		if( _monitorReport )
-			LOG.info("\n"+StatisticMonitor.createReport());
+		_numRuns ++;
 		
 		//reset flags/modifications made by optimizer
 		//TODO reset of hop parallelism constraint (e.g., ba+*)
@@ -741,16 +726,16 @@ public class ParForProgramBlock extends ForProgramBlock
 		//restrict recompilation to thread local memory
 		setMemoryBudget();
 		
+		final LocalTaskQueue<Task> queue = new LocalTaskQueue<>();
+		final Thread[] threads         = new Thread[_numThreads];
+		final LocalParWorker[] workers = new LocalParWorker[_numThreads];
 		try
 		{
 			// Step 1) create task queue and init workers in parallel
 			// (including preparation of update-in-place variables)
-			LocalTaskQueue<Task> queue = new LocalTaskQueue<>();
-			Thread[] threads         = new Thread[_numThreads];
-			LocalParWorker[] workers = new LocalParWorker[_numThreads];
-			IntStream.range(0, _numThreads).parallel().forEach(i -> {
+			IntStream.range(0, _numThreads).forEach(i -> {
 				workers[i] = createParallelWorker( _pwIDs[i], queue, ec, i);
-				threads[i] = new Thread( workers[i] );
+				threads[i] = new Thread( workers[i] , "PARFOR");
 				threads[i].setPriority(Thread.MAX_PRIORITY);
 			});
 			
@@ -762,8 +747,6 @@ public class ParForProgramBlock extends ForProgramBlock
 			long tinit = (long) time.stop();
 			if( DMLScript.STATISTICS )
 				ParForStatistics.incrementInitTime(tinit);
-			if( _monitor ) 
-				StatisticMonitor.putPFStat(_ID, Stat.PARFOR_INIT_PARWRK_T, tinit);
 			
 			// Step 2) create tasks 
 			TaskPartitioner partitioner = createTaskPartitioner(from, to, incr);
@@ -786,16 +769,12 @@ public class ParForProgramBlock extends ForProgramBlock
 				// mark end of task input stream
 				queue.closeInput();
 			}
-			if( _monitor )
-				StatisticMonitor.putPFStat(_ID, Stat.PARFOR_INIT_TASKS_T, time.stop());
 			
 			// Step 3) join all threads (wait for finished work)
+			LineageCacheConfig.setReuseLineageTraces(false); //disable lineage trace reuse
 			for( Thread thread : threads )
 				thread.join();
 			
-			if( _monitor ) 
-				StatisticMonitor.putPFStat(_ID, Stat.PARFOR_WAIT_EXEC_T, time.stop());
-			
 			// Step 4) collecting results from each parallel worker
 			//obtain results and cleanup other intermediates before result merge
 			LocalVariableMap [] localVariables = new LocalVariableMap [_numThreads]; 
@@ -813,6 +792,7 @@ public class ParForProgramBlock extends ForProgramBlock
 				.map(w -> w.getExecutionContext().getLineage())
 				.toArray(Lineage[]::new);
 			mergeLineage(ec, lineages);
+			//LineageCacheConfig.setReuseLineageTraces(true);
 
 			//consolidate results into global symbol table
 			consolidateAndCheckResults( ec, numIterations, numCreatedTasks,
@@ -843,23 +823,18 @@ public class ParForProgramBlock extends ForProgramBlock
 			//remove thread-local memory budget (reset to original budget)
 			//(in finally to prevent error side effects for multiple scripts in one jvm)
 			resetMemoryBudget();
-		
-			if( _monitor ) {
-				StatisticMonitor.putPFStat(_ID, Stat.PARFOR_WAIT_RESULTS_T, time.stop());
-				StatisticMonitor.putPFStat(_ID, Stat.PARFOR_NUMTASKS, numExecutedTasks);
-				StatisticMonitor.putPFStat(_ID, Stat.PARFOR_NUMITERS, numExecutedIterations);
-			}
+
+			if(threads != null) 
+				for(Thread t : threads) 
+					CommonThreadPool.shutdownAsyncPools(t);
 		}
 	}
 
 	private void executeRemoteSparkParFor(ExecutionContext ec, IntObject from, IntObject to, IntObject incr)
 	{
-		Timing time = ( _monitor ? new Timing(true) : null );
-		
 		// Step 0) check and compile to CP (if forced remote parfor)
 		boolean flagForced = false;
-		if( FORCE_CP_ON_REMOTE_SPARK && (_optMode == POptMode.NONE 
-			|| (_optMode == POptMode.CONSTRAINED && _execMode==PExecMode.REMOTE_SPARK)) ) {
+		if( FORCE_CP_ON_REMOTE_SPARK && _execMode==PExecMode.REMOTE_SPARK ) {
 			//tid = 0  because replaced in remote parworker
 			flagForced = checkSparkAndRecompileToCP(0);
 		}
@@ -871,9 +846,6 @@ public class ParForProgramBlock extends ForProgramBlock
 		HashMap<String, byte[]> clsMap = new HashMap<>();
 		String program = ProgramConverter.serializeParForBody(body, clsMap);
 		
-		if( _monitor ) 
-			StatisticMonitor.putPFStat(_ID, Stat.PARFOR_INIT_PARWRK_T, time.stop());
-		
 		// Step 2) create tasks 
 		TaskPartitioner partitioner = createTaskPartitioner(from, to, incr);
 		long numIterations = partitioner.getNumIterations();
@@ -882,27 +854,23 @@ public class ParForProgramBlock extends ForProgramBlock
 		List<Task> tasks = partitioner.createTasks();
 		long numCreatedTasks = tasks.size();
 		
-		if( _monitor )
-			StatisticMonitor.putPFStat(_ID, Stat.PARFOR_INIT_TASKS_T, time.stop());
-		
 		//handle broadcast / export of inputs
 		Set<String> brVars = getBroadcastVariables(ec, _resultVars);
 		exportMatricesToHDFS(ec, brVars);
 		
 		// Step 3) submit Spark parfor job (no lazy evaluation, since collect on result)
+		LineageCacheConfig.setReuseLineageTraces(false); //disable lineage trace reuse
 		boolean topLevelPF = OptimizerUtils.isTopLevelParFor();
 		RemoteParForJobReturn ret = RemoteParForSpark.runJob(_ID, program,
 			clsMap, tasks, ec, brVars, _resultVars, _enableCPCaching, _numThreads, topLevelPF);
 		
-		if( _monitor ) 
-			StatisticMonitor.putPFStat(_ID, Stat.PARFOR_WAIT_EXEC_T, time.stop());
-		
 		// Step 4) collecting results from each parallel worker
 		int numExecutedTasks = ret.getNumExecutedTasks();
 		int numExecutedIterations = ret.getNumExecutedIterations();
 		
 		//lineage maintenance
 		mergeLineage(ec, ret.getLineages());
+		//LineageCacheConfig.setReuseLineageTraces(true);
 		// TODO: remove duplicate lineage items in ec.getLineage()
 		
 		//consolidate results into global symbol table
@@ -910,17 +878,9 @@ public class ParForProgramBlock extends ForProgramBlock
 			numExecutedIterations , numExecutedTasks, ret.getVariables() );
 		if( flagForced ) //see step 0
 			releaseForcedRecompile(0);
-		
-		if( _monitor ) {
-			StatisticMonitor.putPFStat(_ID, Stat.PARFOR_WAIT_RESULTS_T, time.stop());
-			StatisticMonitor.putPFStat(_ID, Stat.PARFOR_NUMTASKS, numExecutedTasks);
-			StatisticMonitor.putPFStat(_ID, Stat.PARFOR_NUMITERS, numExecutedIterations);
-		}
 	}
 	
 	private void executeRemoteSparkParForDP( ExecutionContext ec, IntObject from, IntObject to, IntObject incr ) {
-		Timing time = ( _monitor ? new Timing(true) : null );
-		
 		// Step 0) check and compile to CP (if forced remote parfor)
 		boolean flagForced = checkSparkAndRecompileToCP(0);
 		
@@ -937,18 +897,12 @@ public class ParForProgramBlock extends ForProgramBlock
 		HashMap<String, byte[]> clsMap = new HashMap<>(); 
 		String program = ProgramConverter.serializeParForBody( body, clsMap );
 		
-		if( _monitor ) 
-			StatisticMonitor.putPFStat(_ID, Stat.PARFOR_INIT_PARWRK_T, time.stop());
-		
 		// Step 3) create tasks 
 		TaskPartitioner partitioner = createTaskPartitioner(from, to, incr);
 		String resultFile = constructResultFileName();
 		long numIterations = partitioner.getNumIterations();
 		long numCreatedTasks = numIterations;//partitioner.createTasks().size();
 		
-		if( _monitor )
-			StatisticMonitor.putPFStat(_ID, Stat.PARFOR_INIT_TASKS_T, time.stop());
-		
 		//write matrices to HDFS, except DP matrix which is the input to the RemoteDPParForSpark job
 		exportMatricesToHDFS(ec, CollectionUtils.asSet(_colocatedDPMatrix)); //incl colocated
 		
@@ -956,10 +910,7 @@ public class ParForProgramBlock extends ForProgramBlock
 		RemoteParForJobReturn ret = RemoteDPParForSpark.runJob(
 			_ID, _iterPredVar, _colocatedDPMatrix, program, clsMap, resultFile, inputMatrix,
 			ec, inputDPF, FileFormat.BINARY, _tSparseCol, _enableCPCaching, _numThreads);
-		
-		if( _monitor ) 
-			StatisticMonitor.putPFStat(_ID, Stat.PARFOR_WAIT_EXEC_T, time.stop());
-		
+
 		// Step 5) collecting results from each parallel worker
 		int numExecutedTasks = ret.getNumExecutedTasks();
 		int numExecutedIterations = ret.getNumExecutedIterations();
@@ -971,12 +922,6 @@ public class ParForProgramBlock extends ForProgramBlock
 		if( flagForced ) //see step 0
 			releaseForcedRecompile(0);
 		inputMatrix.unsetPartitioned();
-		
-		if( _monitor ) {
-			StatisticMonitor.putPFStat(_ID, Stat.PARFOR_WAIT_RESULTS_T, time.stop());
-			StatisticMonitor.putPFStat(_ID, Stat.PARFOR_NUMTASKS, numExecutedTasks);
-			StatisticMonitor.putPFStat(_ID, Stat.PARFOR_NUMITERS, numExecutedIterations);
-		}
 	}
 
 	private void handleDataPartitioning( ExecutionContext ec ) 
@@ -1154,32 +1099,34 @@ public class ParForProgramBlock extends ForProgramBlock
 	private void exportMatricesToHDFS(ExecutionContext ec, Set<String> excludeNames)  {
 		ParForStatementBlock sb = (ParForStatementBlock)getStatementBlock();
 		
-		if( LIVEVAR_AWARE_EXPORT && sb != null)
-		{
+		if( LIVEVAR_AWARE_EXPORT && sb != null ) {
 			//optimization to prevent unnecessary export of matrices
 			//export only variables that are read in the body
 			VariableSet varsRead = sb.variablesRead();
 			for (String key : ec.getVariables().keySet() ) {
-				if( varsRead.containsVariable(key) && !excludeNames.contains(key) ) {
-					Data d = ec.getVariable(key);
-					if( d.getDataType().isMatrixOrFrame() )
-						((CacheableData<?>)d).exportData(_replicationExport);
-				}
+				if( varsRead.containsVariable(key) && !excludeNames.contains(key) )
+					exportDataToHDFS(ec.getVariable(key), key);
 			}
 		}
 		else {
 			//export all matrices in symbol table
 			for (String key : ec.getVariables().keySet() ) {
-				if( !excludeNames.contains(key) ) {
-					Data d = ec.getVariable(key);
-					if( d.getDataType().isMatrixOrFrame() )
-						((CacheableData<?>)d).exportData(_replicationExport);
-				}
+				if( !excludeNames.contains(key) )
+					exportDataToHDFS(ec.getVariable(key), key);
 			}
 		}
 	}
+	
+	private void exportDataToHDFS(Data data, String key) {
+		if( data.getDataType().isMatrixOrFrame() )
+			((CacheableData<?>)data).exportData(_replicationExport);
+		if( data.getDataType().isList() ) {
+			for( Data ldata : ((ListObject)data).getData() )
+				exportDataToHDFS(ldata, key);
+		}
+	}
 
-	private void cleanupSharedVariables( ExecutionContext ec, boolean[] varState ) {
+	private void cleanupSharedVariables( ExecutionContext ec, Queue<Boolean> varState ) {
 		//TODO needs as precondition a systematic treatment of persistent read information.
 	}
 	
@@ -1236,7 +1183,7 @@ public class ParForProgramBlock extends ForProgramBlock
 			
 			//create the actual parallel worker
 			ParForBody body = new ParForBody( cpChildBlocks, _resultVars, cpEc );
-			pw = new LocalParWorker( pwID, queue, body, cconf, MAX_RETRYS_ON_ERROR, _monitor );
+			pw = new LocalParWorker( pwID, queue, body, cconf, MAX_RETRYS_ON_ERROR );
 			pw.setFunctionNames(fnNames);
 		}
 		catch(Exception ex) {
@@ -1254,43 +1201,9 @@ public class ParForProgramBlock extends ForProgramBlock
 	 * @param incr ?
 	 * @return task partitioner
 	 */
-	private TaskPartitioner createTaskPartitioner( IntObject from, IntObject to, IntObject incr ) 
-	{
-		TaskPartitioner tp;
-		
-		switch( _taskPartitioner ) {
-			case FIXED:
-				tp = new TaskPartitionerFixedsize(
-					_taskSize, _iterPredVar, from, to, incr);
-				break;
-			case NAIVE:
-				tp = new TaskPartitionerNaive(
-					_taskSize, _iterPredVar, from, to, incr);
-				break;
-			case STATIC:
-				tp = new TaskPartitionerStatic(
-					_taskSize, _numThreads, _iterPredVar, from, to, incr);
-				break;
-			case FACTORING:
-				tp = new TaskPartitionerFactoring(
-					_taskSize,_numThreads, _iterPredVar, from, to, incr);
-				break;
-			case FACTORING_CMIN:
-				//for constrained factoring the tasksize is used as the minimum constraint
-				tp = new TaskPartitionerFactoringCmin(_taskSize,_numThreads, 
-					_taskSize, _iterPredVar, from, to, incr);
-				break;
-
-			case FACTORING_CMAX:
-				//for constrained factoring the tasksize is used as the minimum constraint
-				tp = new TaskPartitionerFactoringCmax(_taskSize,_numThreads, 
-					_taskSize, _iterPredVar, from, to, incr);
-				break;	
-			default:
-				throw new DMLRuntimeException("Undefined task partitioner: '"+_taskPartitioner+"'.");
-		}
-		
-		return tp;
+	private TaskPartitioner createTaskPartitioner( IntObject from, IntObject to, IntObject incr ) {
+		return TaskPartitionerFactory.createTaskPartitioner(
+			_taskPartitioner, from, to, incr, _taskSize, _numThreads, _iterPredVar);
 	}
 	
 	/**
@@ -1324,8 +1237,8 @@ public class ParForProgramBlock extends ForProgramBlock
 		return dp;
 	}
 
-	private ResultMerge<?> createResultMerge( PResultMerge prm,
-		CacheableData<?> out, CacheableData<?>[] in, String fname, boolean accum, ExecutionContext ec ) 
+	public static ResultMerge<?> createResultMerge( PResultMerge prm,
+		CacheableData<?> out, CacheableData<?>[] in, String fname, boolean accum, int numThreads, ExecutionContext ec ) 
 	{
 		ResultMerge<?> rm;
 		
@@ -1347,7 +1260,7 @@ public class ParForProgramBlock extends ForProgramBlock
 					rm = new ResultMergeLocalAutomatic( (MatrixObject)out, (MatrixObject[])in, fname, accum );
 					break;
 				case REMOTE_SPARK:
-					int numMap = Math.max(_numThreads,
+					int numMap = Math.max(numThreads,
 						SparkExecutionContext.getDefaultParallelism(true));
 					int numRed = numMap; //equal map/reduce
 					rm = new ResultMergeRemoteSpark( (MatrixObject)out,
@@ -1385,8 +1298,8 @@ public class ParForProgramBlock extends ForProgramBlock
 		}
 		
 		//try recompile Spark instructions to CP
-		HashSet<String> fnStack = new HashSet<>();
-		Recompiler.recompileProgramBlockHierarchy2Forced(_childBlocks, tid, fnStack, ExecType.CP);
+		Recompiler.recompileProgramBlockHierarchy2Forced(
+			_childBlocks, tid, new HashSet<>(), ExecType.CP);
 		return true;
 	}
 
@@ -1413,9 +1326,14 @@ public class ParForProgramBlock extends ForProgramBlock
 		}
 	}
 
-	private void consolidateAndCheckResults(ExecutionContext ec, long expIters, long expTasks, long numIters, long numTasks, LocalVariableMap [] results) 
-	{
+	private void consolidateAndCheckResults(ExecutionContext ec, final long expIters, final long expTasks,
+		final long numIters, final long numTasks, LocalVariableMap[] results) {
 		Timing time = new Timing(true);
+
+		//check expected counters
+		if( numTasks != expTasks || numIters !=expIters ) //consistency check
+			throw new DMLRuntimeException("PARFOR: Number of executed tasks does not match the number of created tasks: tasks "+numTasks+"/"+expTasks+", iters "+numIters+"/"+expIters+".");
+	
 		
 		//result merge
 		if( checkParallelRemoteResultMerge() )
@@ -1469,7 +1387,8 @@ public class ParForProgramBlock extends ForProgramBlock
 					CacheableData<?>[] in = (dat instanceof MatrixObject) ?
 						tmp.toArray(MatrixObject[]::new) : tmp.toArray(FrameObject[]::new);
 					String fname = constructResultMergeFileName();
-					ResultMerge<?> rm = createResultMerge(_resultMerge, out, in, fname, var._isAccum, ec);
+					ResultMerge<?> rm = createResultMerge(_resultMerge,
+						out, in, fname, var._isAccum, _numThreads, ec);
 					CacheableData<?> outNew = USE_PARALLEL_RESULT_MERGE ?
 						rm.executeParallelMerge(_numThreads) :
 						rm.executeSerialMerge();
@@ -1514,10 +1433,7 @@ public class ParForProgramBlock extends ForProgramBlock
 		if( CREATE_UNSCOPED_RESULTVARS && sb != null && ec.getVariables() != null ) //sb might be null for nested parallelism
 			createEmptyUnscopedVariables( ec.getVariables(), sb );
 		
-		//check expected counters
-		if( numTasks != expTasks || numIters !=expIters ) //consistency check
-			throw new DMLRuntimeException("PARFOR: Number of executed tasks does not match the number of created tasks: tasks "+numTasks+"/"+expTasks+", iters "+numIters+"/"+expIters+".");
-	
+			
 		if( DMLScript.STATISTICS )
 			ParForStatistics.incrementMergeTime((long) time.stop());
 	}
@@ -1543,12 +1459,7 @@ public class ParForProgramBlock extends ForProgramBlock
 			_ID = IDHandler.concatIntIDsToLong(_IDPrefix, (int)_pfIDSeq.getNextID());	
 	}
 	
-	/**
-	 * TODO rework id handling in order to enable worker reuse
-	 * 
-	 */
-	private void setLocalParWorkerIDs()
-	{
+	private void setLocalParWorkerIDs() {
 		if( _numThreads<=0 )
 			return;
 		
@@ -1561,9 +1472,6 @@ public class ParForProgramBlock extends ForProgramBlock
 				_pwIDs[i] = _pwIDSeq.getNextID();
 			else
 				_pwIDs[i] = IDHandler.concatIntIDsToLong(_IDPrefix,(int)_pwIDSeq.getNextID());
-			
-			if( _monitor ) 
-				StatisticMonitor.putPfPwMapping(_ID, _pwIDs[i]);
 		}
 	}
 	
@@ -1690,7 +1598,8 @@ public class ParForProgramBlock extends ForProgramBlock
 					
 					String fname = constructResultMergeFileName();
 				
-					ResultMerge<?> rm = createResultMerge(_resultMerge, out, in, fname, var._isAccum, _ec);
+					ResultMerge<?> rm = createResultMerge(_resultMerge,
+						out, in, fname, var._isAccum, _numThreads, _ec);
 					CacheableData<?> outNew;
 					if( USE_PARALLEL_RESULT_MERGE )
 						outNew = rm.executeParallelMerge( _numThreads );
