@@ -20,8 +20,10 @@
 package org.apache.sysds.runtime.data;
 
 import java.io.Serializable;
+import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Iterator;
+import java.util.List;
 
 import org.apache.sysds.runtime.matrix.data.IJV;
 
@@ -46,9 +48,12 @@ public abstract class SparseBlock implements Serializable, Block
 	protected static final double RESIZE_FACTOR2 = 1.1; //factor after reaching est nnz
 	
 	public enum Type {
-		MCSR,
-		CSR,
-		COO,
+		COO,  // coordinate
+		CSR,  // compressed sparse rows
+		CSC,  // compressed sparse column
+		DCSR, // double compressed sparse rows
+		MCSR, // modified compressed sparse rows (update-friendly)
+		MCSC, // modified compressed sparse column
 	}
 	
 	
@@ -270,6 +275,12 @@ public abstract class SparseBlock implements Serializable, Block
 
 	public abstract boolean checkValidity(int rlen, int clen, long nnz, boolean strict);
 
+	/**
+	 * Computes the exact size in memory of the materialized block
+	 * @return the exact size in memory
+	 */
+	public abstract long getExactSizeInMemory();
+	
 	////////////////////////
 	//obtain indexes/values/positions
 	
@@ -422,9 +433,6 @@ public abstract class SparseBlock implements Serializable, Block
 	/**
 	 * Get values of row r in the format of a sparse row. 
 	 * 
-	 * NOTE: This method exists for incremental runtime integration and might
-	 * be deleted in the future.
-	 * 
 	 * @param r  row index starting at 0
 	 * @return values of row r as a sparse row
 	 */
@@ -474,12 +482,13 @@ public abstract class SparseBlock implements Serializable, Block
 	 * (note that NaN==NaN yields false).
 	 * 
 	 * @param pattern checked pattern
+	 * @param rl row lower bound (inclusive)
+	 * @param ru row upper bound (exclusive)
 	 * @return true if pattern appears at least once, otherwise false
 	 */
-	public boolean contains(double pattern) {
+	public boolean contains(double pattern, int rl, int ru) {
 		boolean NaNpattern = Double.isNaN(pattern);
-		int rlen = numRows();
-		for(int i=0; i<rlen; i++) {
+		for(int i=rl; i<ru; i++) {
 			if( isEmpty(i) ) continue;
 			int apos = pos(i);
 			int alen = size(i);
@@ -491,6 +500,26 @@ public abstract class SparseBlock implements Serializable, Block
 		return false;
 	}
 	
+	public List<Integer> contains(double[] pattern, boolean earlyAbort) {
+		List<Integer> ret = new ArrayList<>();
+		int rlen = numRows();
+		for( int i=0; i<rlen; i++ ) {
+			int apos = pos(i);
+			int alen = size(i);
+			int[] aix = indexes(i);
+			double[] avals = values(i);
+			boolean lret = true;
+			//safe comparison on long representations, incl NaN
+			for(int k=apos; k<apos+alen & !lret; k++)
+				lret &= Double.compare(avals[k], pattern[aix[k]]) == 0;
+			if( lret )
+				ret.add(i);
+			if(earlyAbort && ret.size()>0)
+				return ret;
+		}
+		return ret;
+	}
+	
 	////////////////////////
 	//iterators
 	
@@ -533,6 +562,48 @@ public abstract class SparseBlock implements Serializable, Block
 		return new SparseBlockIterator(rl, Math.min(ru,numRows()));
 	}
 	
+	/**
+	 * Get a non-zero iterator over the subblock [rl/cl, ru/cu). Note that
+	 * the returned IJV object is reused across next calls and should 
+	 * be directly consumed or deep copied. 
+	 * 
+	 * @param rl   inclusive lower row index starting at 0
+	 * @param ru   exclusive upper row index starting at 0
+	 * @param cl   inclusive lower column index starting at 0
+	 * @param cu   exclusive upper column index starting at 0
+	 * @return IJV iterator
+	 */
+	public Iterator<IJV> getIterator(int rl, int ru, int cl, int cu) {
+		//default generic iterator, override if necessary
+		return new SparseBlockIterator(rl, Math.min(ru,numRows()), cl, cu);
+	}
+
+	/**
+	 * Get an iterator over the indices of non-empty rows within the entire sparse block.
+	 * This iterator facilitates traversal over rows that contain at least one non-zero element,
+	 * skipping entirely zero rows. The returned integers represent the indexes of non-empty rows.
+	 *
+	 * @return iterable
+	 */
+	public Iterable<Integer> getNonEmptyRows() {
+		return new SparseNonEmptyRowIterable(0, numRows());
+	}
+
+	/**
+	 * Get an iterator over the indices of non-zero rows within the sub-block [rl,ru).
+	 * This iterator facilitates traversal over rows that contain at least one non-zero element,
+	 * skipping entirely zero rows. The returned integers represent the indexes of non-empty rows.
+	 *
+	 * @param rl inclusive lower row index starting at 0
+	 * @param ru exclusive upper row index starting at 0
+	 * @return iterable
+	 */
+	public Iterable<Integer> getNonEmptyRows(int rl, int ru) {
+		return new SparseNonEmptyRowIterable(rl, ru);
+	}
+	
+	public abstract Iterator<Integer> getNonEmptyRowsIterator(int rl, int ru);
+	
 	@Override 
 	public abstract String toString();
 	
@@ -639,19 +710,29 @@ public abstract class SparseBlock implements Serializable, Block
 		private int _curColIx = -1; //current col index pos
 		private int[] _curIndexes = null; //current col indexes
 		private double[] _curValues = null; //current col values
- 		private boolean _noNext = false; //end indicator		
+ 		private boolean _noNext = false; //end indicator
 		private IJV retijv = new IJV(); //reuse output tuple
+		private int _cl = 0;
+		private int _cu = Integer.MAX_VALUE;
 
 		protected SparseBlockIterator(int ru) {
 			_rlen = ru;
 			_curRow = 0;
-			findNextNonZeroRow();
+			findNextNonZeroRow(0);
 		}
 		
 		protected SparseBlockIterator(int rl, int ru) {
 			_rlen = ru;
 			_curRow = rl;
-			findNextNonZeroRow();
+			findNextNonZeroRow(0);
+		}
+		
+		protected SparseBlockIterator(int rl, int ru, int cl, int cu) {
+			_rlen = ru;
+			_curRow = rl;
+			_cl = cl;
+			_cu = cu;
+			findNextNonZeroRow(cl);
 		}
 		
 		@Override
@@ -662,14 +743,12 @@ public abstract class SparseBlock implements Serializable, Block
 		@Override
 		public IJV next( ) {
 			retijv.set(_curRow, _curIndexes[_curColIx], _curValues[_curColIx]);
-			
-			//NOTE: no preincrement on curcolix to avoid OpenJDK8 escape analysis bug, encountered 
-			//with tests SparsityRecompileTest/SparsityFunctionRecompileTest on parfor local result merge
-			if( _curColIx < pos(_curRow)+size(_curRow)-1 ) 
+			if( _curColIx < pos(_curRow)+size(_curRow)-1 && _curIndexes[_curColIx+1] < _cu ) { 
 				_curColIx++;
+			}
 			else {
 				_curRow++;
-				findNextNonZeroRow();	
+				findNextNonZeroRow(_cl);
 			}
 
 			return retijv;
@@ -678,22 +757,41 @@ public abstract class SparseBlock implements Serializable, Block
 		@Override
 		public void remove() {
 			throw new RuntimeException("SparseBlockIterator is unsupported!");
-		}		
+		}
 		
 		/**
 		 * Moves cursor to next non-zero value or indicates that no more 
 		 * values are available.
 		 */
-		private void findNextNonZeroRow() {
-			while( _curRow<_rlen && isEmpty(_curRow))
+		private void findNextNonZeroRow(int cl) {
+			while( _curRow<_rlen && (isEmpty(_curRow) 
+				|| (cl>0 && posFIndexGTE(_curRow, cl) < 0)) )
 				_curRow++;
 			if(_curRow >= _rlen)
 				_noNext = true;
 			else {
-				_curColIx = pos(_curRow);
+				_curColIx = (cl==0) ? 
+					pos(_curRow) : posFIndexGTE(_curRow, cl);
 				_curIndexes = indexes(_curRow); 
 				_curValues = values(_curRow);
 			}
 		}
 	}
+	
+	//generic iterable for use in enhanced for loops: for(int i : s.getNonEmptyRows())
+	private class SparseNonEmptyRowIterable implements Iterable<Integer> {
+		private final int _rl; //row lower
+		private final int _ru; //row upper
+
+		protected SparseNonEmptyRowIterable(int rl, int ru) {
+			_rl = rl;
+			_ru  =ru;
+		}
+
+		@Override
+		public Iterator<Integer> iterator() {
+			//use specialized non-empty row iterators of sparse blocks
+			return getNonEmptyRowsIterator(_rl, _ru);
+		}
+	}
 }
