@@ -19,10 +19,19 @@
 
 package org.apache.sysds.runtime.instructions.cp;
 
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.HashMap;
+import java.util.LinkedHashMap;
+import java.util.List;
+import java.util.stream.Collectors;
+import java.util.stream.IntStream;
+
 import org.apache.commons.lang3.tuple.Pair;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.apache.sysds.api.DMLScript;
+import org.apache.sysds.common.Opcodes;
 import org.apache.sysds.common.Types.DataType;
 import org.apache.sysds.common.Types.ValueType;
 import org.apache.sysds.hops.OptimizerUtils;
@@ -30,16 +39,20 @@ import org.apache.sysds.lops.Lop;
 import org.apache.sysds.parser.ParameterizedBuiltinFunctionExpression;
 import org.apache.sysds.parser.Statement;
 import org.apache.sysds.runtime.DMLRuntimeException;
-import org.apache.sysds.runtime.controlprogram.caching.*;
+import org.apache.sysds.runtime.controlprogram.caching.CacheBlock;
+import org.apache.sysds.runtime.controlprogram.caching.CacheableData;
+import org.apache.sysds.runtime.controlprogram.caching.FrameObject;
+import org.apache.sysds.runtime.controlprogram.caching.MatrixObject;
+import org.apache.sysds.runtime.controlprogram.caching.TensorObject;
 import org.apache.sysds.runtime.controlprogram.context.ExecutionContext;
 import org.apache.sysds.runtime.controlprogram.context.ExecutionContextFactory;
 import org.apache.sysds.runtime.data.TensorBlock;
+import org.apache.sysds.runtime.frame.data.FrameBlock;
 import org.apache.sysds.runtime.functionobjects.ParameterizedBuiltin;
 import org.apache.sysds.runtime.functionobjects.ValueFunction;
 import org.apache.sysds.runtime.instructions.InstructionUtils;
 import org.apache.sysds.runtime.lineage.LineageItem;
 import org.apache.sysds.runtime.lineage.LineageItemUtils;
-import org.apache.sysds.runtime.matrix.data.FrameBlock;
 import org.apache.sysds.runtime.matrix.data.MatrixBlock;
 import org.apache.sysds.runtime.matrix.operators.Operator;
 import org.apache.sysds.runtime.matrix.operators.SimpleOperator;
@@ -53,10 +66,7 @@ import org.apache.sysds.runtime.transform.tokenize.Tokenizer;
 import org.apache.sysds.runtime.transform.tokenize.TokenizerFactory;
 import org.apache.sysds.runtime.util.AutoDiff;
 import org.apache.sysds.runtime.util.DataConverter;
-
-import java.util.*;
-import java.util.stream.Collectors;
-import java.util.stream.IntStream;
+import org.apache.sysds.utils.stats.InfrastructureAnalyzer;
 
 public class ParameterizedBuiltinCPInstruction extends ComputationCPInstruction {
 	private static final Log LOG = LogFactory.getLog(ParameterizedBuiltinCPInstruction.class.getName());
@@ -108,21 +118,21 @@ public class ParameterizedBuiltinCPInstruction extends ComputationCPInstruction
 
 		// determine the appropriate value function
 		ValueFunction func = null;
-		if(opcode.equalsIgnoreCase("cdf")) {
+		if(opcode.equalsIgnoreCase(Opcodes.CDF.toString())) {
 			if(paramsMap.get("dist") == null)
 				throw new DMLRuntimeException("Invalid distribution: " + str);
 			func = ParameterizedBuiltin.getParameterizedBuiltinFnObject(opcode, paramsMap.get("dist"));
 			// Determine appropriate Function Object based on opcode
 			return new ParameterizedBuiltinCPInstruction(new SimpleOperator(func), paramsMap, out, opcode, str);
 		}
-		else if(opcode.equalsIgnoreCase("invcdf")) {
+		else if(opcode.equalsIgnoreCase(Opcodes.INVCDF.toString())) {
 			if(paramsMap.get("dist") == null)
 				throw new DMLRuntimeException("Invalid distribution: " + str);
 			func = ParameterizedBuiltin.getParameterizedBuiltinFnObject(opcode, paramsMap.get("dist"));
 			// Determine appropriate Function Object based on opcode
 			return new ParameterizedBuiltinCPInstruction(new SimpleOperator(func), paramsMap, out, opcode, str);
 		}
-		else if(opcode.equalsIgnoreCase("groupedagg")) {
+		else if(opcode.equalsIgnoreCase(Opcodes.GROUPEDAGG.toString())) {
 			// check for mandatory arguments
 			String fnStr = paramsMap.get("fn");
 			if(fnStr == null)
@@ -136,18 +146,19 @@ public class ParameterizedBuiltinCPInstruction extends ComputationCPInstruction
 			Operator op = InstructionUtils.parseGroupedAggOperator(fnStr, paramsMap.get("order"));
 			return new ParameterizedBuiltinCPInstruction(op, paramsMap, out, opcode, str);
 		}
-		else if(opcode.equalsIgnoreCase("rmempty") || opcode.equalsIgnoreCase("replace") ||
-			opcode.equalsIgnoreCase("rexpand") || opcode.equalsIgnoreCase("lowertri") ||
-			opcode.equalsIgnoreCase("uppertri")) {
+		else if(opcode.equalsIgnoreCase(Opcodes.RMEMPTY.toString()) || opcode.equalsIgnoreCase(Opcodes.REPLACE.toString()) ||
+			opcode.equalsIgnoreCase(Opcodes.REXPAND.toString()) || opcode.equalsIgnoreCase(Opcodes.LOWERTRI.toString()) ||
+			opcode.equalsIgnoreCase(Opcodes.UPPERTRI.toString()) ) {
 			func = ParameterizedBuiltin.getParameterizedBuiltinFnObject(opcode);
 			return new ParameterizedBuiltinCPInstruction(new SimpleOperator(func), paramsMap, out, opcode, str);
 		}
-		else if(opcode.equals("transformapply") || opcode.equals("transformdecode") ||
-			opcode.equals("transformcolmap") || opcode.equals("transformmeta") || opcode.equals("tokenize") ||
-			opcode.equals("toString") || opcode.equals("nvlist") || opcode.equals("autoDiff")) {
+		else if(opcode.equals(Opcodes.TRANSFORMAPPLY.toString()) || opcode.equals(Opcodes.TRANSFORMDECODE.toString())
+			|| opcode.equalsIgnoreCase(Opcodes.CONTAINS.toString()) || opcode.equals(Opcodes.TRANSFORMCOLMAP.toString())
+			|| opcode.equals(Opcodes.TRANSFORMMETA.toString()) || opcode.equals(Opcodes.TOKENIZE.toString())
+			|| opcode.equals(Opcodes.TOSTRING.toString()) || opcode.equals(Opcodes.NVLIST.toString()) || opcode.equals(Opcodes.AUTODIFF.toString())) {
 			return new ParameterizedBuiltinCPInstruction(null, paramsMap, out, opcode, str);
 		}
-		else if("paramserv".equals(opcode)) {
+		else if(Opcodes.PARAMSERV.toString().equals(opcode)) {
 			return new ParamservBuiltinCPInstruction(null, paramsMap, out, opcode, str);
 		}
 		else {
@@ -160,26 +171,26 @@ public class ParameterizedBuiltinCPInstruction extends ComputationCPInstruction
 	public void processInstruction(ExecutionContext ec) {
 		String opcode = getOpcode();
 		ScalarObject sores = null;
-		if(opcode.equalsIgnoreCase("cdf")) {
+		if(opcode.equalsIgnoreCase(Opcodes.CDF.toString())) {
 			SimpleOperator op = (SimpleOperator) _optr;
 			double result = op.fn.execute(params);
 			sores = new DoubleObject(result);
 			ec.setScalarOutput(output.getName(), sores);
 		}
-		else if(opcode.equalsIgnoreCase("invcdf")) {
+		else if(opcode.equalsIgnoreCase(Opcodes.INVCDF.toString())) {
 			SimpleOperator op = (SimpleOperator) _optr;
 			double result = op.fn.execute(params);
 			sores = new DoubleObject(result);
 			ec.setScalarOutput(output.getName(), sores);
 		}
-		else if(opcode.equalsIgnoreCase("autoDiff"))
+		else if(opcode.equalsIgnoreCase(Opcodes.AUTODIFF.toString()))
 		{
 			ArrayList<Data> lineage = (ArrayList<Data>) ec.getListObject(params.get("lineage")).getData();
 			MatrixObject mo = ec.getMatrixObject(params.get("output"));
 			ListObject diffs = AutoDiff.getBackward(mo, lineage, ExecutionContextFactory.createContext());
 			ec.setVariable(output.getName(), diffs);
 		}
-		else if(opcode.equalsIgnoreCase("groupedagg")) {
+		else if(opcode.equalsIgnoreCase(Opcodes.GROUPEDAGG.toString())) {
 			// acquire locks
 			MatrixBlock target = ec.getMatrixInput(params.get(Statement.GAGG_TARGET));
 			MatrixBlock groups = ec.getMatrixInput(params.get(Statement.GAGG_GROUPS));
@@ -205,7 +216,7 @@ public class ParameterizedBuiltinCPInstruction extends ComputationCPInstruction
 				ec.releaseMatrixInput(params.get(Statement.GAGG_WEIGHTS));
 
 		}
-		else if(opcode.equalsIgnoreCase("rmempty")) {
+		else if(opcode.equalsIgnoreCase(Opcodes.RMEMPTY.toString())) {
 			String margin = params.get("margin");
 			if(!(margin.equals("rows") || margin.equals("cols")))
 				throw new DMLRuntimeException("Unspupported margin identifier '" + margin + "'.");
@@ -226,16 +237,34 @@ public class ParameterizedBuiltinCPInstruction extends ComputationCPInstruction
 
 				// compute the result
 				boolean emptyReturn = Boolean.parseBoolean(params.get("empty.return").toLowerCase());
-				MatrixBlock soresBlock = target.removeEmptyOperations(new MatrixBlock(), margin.equals("rows"), emptyReturn, select);
+				MatrixBlock ret = target.removeEmptyOperations(new MatrixBlock(), margin.equals("rows"), emptyReturn, select);
 
 				// release locks
-				ec.setMatrixOutput(output.getName(), soresBlock);
+				if( target == ret ) //short-circuit (avoid buffer pool pollution)
+					ec.setVariable(output.getName(), ec.getVariable(params.get("target")));
+				else
+					ec.setMatrixOutput(output.getName(), ret);
 				ec.releaseMatrixInput(params.get("target"));
 				if(params.containsKey("select"))
 					ec.releaseMatrixInput(params.get("select"));
 			}
 		}
-		else if(opcode.equalsIgnoreCase("replace")) {
+		else if(opcode.equalsIgnoreCase(Opcodes.CONTAINS.toString())) {
+			String varName = params.get("target");
+			int k = Integer.parseInt(params.get("k")); //num threads
+			MatrixBlock target = ec.getMatrixInput(varName);
+			Data pattern = ec.getVariable(params.get("pattern"));
+			if( pattern == null ) //literal
+				pattern = ScalarObjectFactory.createScalarObject(ValueType.FP64, params.get("pattern"));
+			boolean ret = pattern.getDataType().isScalar() ?
+				target.containsValue(((ScalarObject)pattern).getDoubleValue(), k) : 
+				(target.containsVector(((MatrixObject)pattern).acquireRead(), true).size()>0);
+			ec.releaseMatrixInput(varName);
+			if(!pattern.getDataType().isScalar())
+				ec.releaseMatrixInput(params.get("pattern"));
+			ec.setScalarOutput(output.getName(), new BooleanObject(ret));
+		}
+		else if(opcode.equalsIgnoreCase(Opcodes.REPLACE.toString())) {
 			if(ec.isFrameObject(params.get("target"))){
 				FrameBlock target = ec.getFrameInput(params.get("target"));
 				String pattern = params.get("pattern");
@@ -248,25 +277,25 @@ public class ParameterizedBuiltinCPInstruction extends ComputationCPInstruction
 				MatrixBlock target = targetObj.acquireRead();
 				double pattern = Double.parseDouble(params.get("pattern"));
 				double replacement = Double.parseDouble(params.get("replacement"));
-				MatrixBlock ret = target.replaceOperations(new MatrixBlock(), pattern, replacement);
+				MatrixBlock ret = target.replaceOperations(new MatrixBlock(), pattern, replacement, 
+					InfrastructureAnalyzer.getLocalParallelism());
 				if( ret == target ) //shallow copy (avoid bufferpool pollution)
 					ec.setVariable(output.getName(), targetObj);
 				else
 					ec.setMatrixOutput(output.getName(), ret);
 				targetObj.release();
 			}
-			
 		}
-		else if(opcode.equals("lowertri") || opcode.equals("uppertri")) {
+		else if(opcode.equals(Opcodes.LOWERTRI.toString()) || opcode.equals(Opcodes.UPPERTRI.toString())) {
 			MatrixBlock target = ec.getMatrixInput(params.get("target"));
-			boolean lower = opcode.equals("lowertri");
+			boolean lower = opcode.equals(Opcodes.LOWERTRI.toString());
 			boolean diag = Boolean.parseBoolean(params.get("diag"));
 			boolean values = Boolean.parseBoolean(params.get("values"));
 			MatrixBlock ret = target.extractTriangular(new MatrixBlock(), lower, diag, values);
 			ec.setMatrixOutput(output.getName(), ret);
 			ec.releaseMatrixInput(params.get("target"));
 		}
-		else if(opcode.equalsIgnoreCase("rexpand")) {
+		else if(opcode.equalsIgnoreCase(Opcodes.REXPAND.toString())) {
 			// acquire locks
 			MatrixBlock target = ec.getMatrixInput(params.get("target"));
 
@@ -282,36 +311,39 @@ public class ParameterizedBuiltinCPInstruction extends ComputationCPInstruction
 			ec.setMatrixOutput(output.getName(), ret);
 			ec.releaseMatrixInput(params.get("target"));
 		}
-		else if(opcode.equalsIgnoreCase("tokenize")) {
+		else if(opcode.equalsIgnoreCase(Opcodes.TOKENIZE.toString())) {
 			// acquire locks
 			FrameBlock data = ec.getFrameInput(params.get("target"));
 
 			// compute tokenizer
 			Tokenizer tokenizer = TokenizerFactory.createTokenizer(getParameterMap().get("spec"),
 				Integer.parseInt(getParameterMap().get("max_tokens")));
-			FrameBlock fbout = tokenizer.tokenize(data, new FrameBlock(tokenizer.getSchema()));
+			FrameBlock fbout = tokenizer.tokenize(data, OptimizerUtils.getTokenizeNumThreads());
 
 			// release locks
 			ec.setFrameOutput(output.getName(), fbout);
 			ec.releaseFrameInput(params.get("target"));
 		}
-		else if(opcode.equalsIgnoreCase("transformapply")) {
+		else if(opcode.equalsIgnoreCase(Opcodes.TRANSFORMAPPLY.toString())) {
 			// acquire locks
 			FrameBlock data = ec.getFrameInput(params.get("target"));
 			FrameBlock meta = ec.getFrameInput(params.get("meta"));
+			MatrixBlock embeddings = params.get("embedding") != null ? ec.getMatrixInput(params.get("embedding")) : null;
 			String[] colNames = data.getColumnNames();
 
 			// compute transformapply
 			MultiColumnEncoder encoder = EncoderFactory
-				.createEncoder(params.get("spec"), colNames, data.getNumColumns(), meta);
+				.createEncoder(params.get("spec"), colNames, data.getNumColumns(), meta, embeddings);
 			MatrixBlock mbout = encoder.apply(data, OptimizerUtils.getTransformNumThreads());
 
 			// release locks
 			ec.setMatrixOutput(output.getName(), mbout);
 			ec.releaseFrameInput(params.get("target"));
 			ec.releaseFrameInput(params.get("meta"));
+			if(params.get("embedding") != null)
+				ec.releaseMatrixInput(params.get("embedding"));
 		}
-		else if(opcode.equalsIgnoreCase("transformdecode")) {
+		else if(opcode.equalsIgnoreCase(Opcodes.TRANSFORMDECODE.toString())) {
 			// acquire locks
 			MatrixBlock data = ec.getMatrixInput(params.get("target"));
 			FrameBlock meta = ec.getFrameInput(params.get("meta"));
@@ -328,21 +360,21 @@ public class ParameterizedBuiltinCPInstruction extends ComputationCPInstruction
 			ec.releaseMatrixInput(params.get("target"));
 			ec.releaseFrameInput(params.get("meta"));
 		}
-		else if(opcode.equalsIgnoreCase("transformcolmap")) {
+		else if(opcode.equalsIgnoreCase(Opcodes.TRANSFORMCOLMAP.toString())) {
 			// acquire locks
 			FrameBlock meta = ec.getFrameInput(params.get("target"));
 			String[] colNames = meta.getColumnNames();
 
 			// compute transformapply
 			MultiColumnEncoder encoder = EncoderFactory
-				.createEncoder(params.get("spec"), colNames, meta.getNumColumns(), null);
+				.createEncoder(params.get("spec"), colNames, meta.getNumColumns(), null, null);
 			MatrixBlock mbout = encoder.getColMapping(meta);
 
 			// release locks
 			ec.setMatrixOutput(output.getName(), mbout);
 			ec.releaseFrameInput(params.get("target"));
 		}
-		else if(opcode.equalsIgnoreCase("transformmeta")) {
+		else if(opcode.equalsIgnoreCase(Opcodes.TRANSFORMMETA.toString())) {
 			// get input spec and path
 			String spec = getParameterMap().get("spec");
 			String path = getParameterMap().get(ParameterizedBuiltinFunctionExpression.TF_FN_PARAM_MTD);
@@ -360,7 +392,7 @@ public class ParameterizedBuiltinCPInstruction extends ComputationCPInstruction
 			// release locks
 			ec.setFrameOutput(output.getName(), meta);
 		}
-		else if(opcode.equalsIgnoreCase("toString")) {
+		else if(opcode.equalsIgnoreCase(Opcodes.TOSTRING.toString())) {
 			// handle input parameters
 			int rows = (getParam("rows") != null) ? Integer.parseInt(getParam("rows")) : TOSTRING_MAXROWS;
 			int cols = (getParam("cols") != null) ? Integer.parseInt(getParam("cols")) : TOSTRING_MAXCOLS;
@@ -391,14 +423,7 @@ public class ParameterizedBuiltinCPInstruction extends ComputationCPInstruction
 			}
 			else if(cacheData instanceof ListObject) {
 				out = DataConverter.toString((ListObject) cacheData,
-					rows,
-					cols,
-					sparse,
-					separator,
-					lineSeparator,
-					rows,
-					cols,
-					decimal);
+					rows, cols, sparse, separator, lineSeparator, rows, cols, decimal);
 			}
 			else {
 				throw new DMLRuntimeException("toString only converts "
@@ -409,7 +434,7 @@ public class ParameterizedBuiltinCPInstruction extends ComputationCPInstruction
 			}
 			ec.setScalarOutput(output.getName(), new StringObject(out));
 		}
-		else if(opcode.equals("nvlist")) {
+		else if(opcode.equals(Opcodes.NVLIST.toString())) {
 			// obtain all input data objects and names in insertion order
 			List<Data> data = params.values().stream()
 				.map(d -> ec.containsVariable(d) ? ec.getVariable(d) :
@@ -439,7 +464,7 @@ public class ParameterizedBuiltinCPInstruction extends ComputationCPInstruction
 		}
 	}
 
-	private void warnOnTrunction(CacheBlock data, int rows, int cols) {
+	private void warnOnTrunction(CacheBlock<?> data, int rows, int cols) {
 		// warn on truncation because users might not be aware and use toString for verification
 		if((getParam("rows") == null && data.getNumRows() > rows) ||
 			(getParam("cols") == null && data.getNumColumns() > cols)) {
@@ -466,7 +491,13 @@ public class ParameterizedBuiltinCPInstruction extends ComputationCPInstruction
 	@Override
 	public Pair<String, LineageItem> getLineageItem(ExecutionContext ec) {
 		String opcode = getOpcode();
-		if(opcode.equalsIgnoreCase("groupedagg")) {
+		if(opcode.equalsIgnoreCase(Opcodes.CONTAINS.toString())) {
+			CPOperand target = getTargetOperand();
+			CPOperand pattern = getFP64Literal("pattern");
+			return Pair.of(output.getName(),
+				new LineageItem(getOpcode(), LineageItemUtils.getLineage(ec, target, pattern)));
+		}
+		else if(opcode.equalsIgnoreCase(Opcodes.GROUPEDAGG.toString())) {
 			CPOperand target = getTargetOperand();
 			CPOperand groups = new CPOperand(params.get(Statement.GAGG_GROUPS), ValueType.FP64, DataType.MATRIX);
 			String wt = params.containsKey(Statement.GAGG_WEIGHTS) ? params.get(Statement.GAGG_WEIGHTS) : String
@@ -479,7 +510,7 @@ public class ParameterizedBuiltinCPInstruction extends ComputationCPInstruction
 			return Pair.of(output.getName(),
 				new LineageItem(getOpcode(), LineageItemUtils.getLineage(ec, target, groups, weights, fn, ngroups)));
 		}
-		else if(opcode.equalsIgnoreCase("rmempty")) {
+		else if(opcode.equalsIgnoreCase(Opcodes.RMEMPTY.toString())) {
 			CPOperand target = getTargetOperand();
 			CPOperand margin = getStringLiteral("margin");
 			String sl = params.containsKey("select") ? params.get("select") : String.valueOf(-1);
@@ -487,14 +518,14 @@ public class ParameterizedBuiltinCPInstruction extends ComputationCPInstruction
 			return Pair.of(output.getName(),
 				new LineageItem(getOpcode(), LineageItemUtils.getLineage(ec, target, margin, select)));
 		}
-		else if(opcode.equalsIgnoreCase("replace")) {
+		else if(opcode.equalsIgnoreCase(Opcodes.REPLACE.toString())) {
 			CPOperand target = getTargetOperand();
 			CPOperand pattern = getFP64Literal("pattern");
 			CPOperand replace = getFP64Literal("replacement");
 			return Pair.of(output.getName(),
 				new LineageItem(getOpcode(), LineageItemUtils.getLineage(ec, target, pattern, replace)));
 		}
-		else if(opcode.equalsIgnoreCase("rexpand")) {
+		else if(opcode.equalsIgnoreCase(Opcodes.REXPAND.toString())) {
 			CPOperand target = getTargetOperand();
 			CPOperand max = getFP64Literal("max");
 			CPOperand dir = getStringLiteral("dir");
@@ -503,22 +534,22 @@ public class ParameterizedBuiltinCPInstruction extends ComputationCPInstruction
 			return Pair.of(output.getName(),
 				new LineageItem(getOpcode(), LineageItemUtils.getLineage(ec, target, max, dir, cast, ignore)));
 		}
-		else if(opcode.equalsIgnoreCase("lowertri") || opcode.equalsIgnoreCase("uppertri")) {
+		else if(opcode.equalsIgnoreCase(Opcodes.LOWERTRI.toString()) || opcode.equalsIgnoreCase(Opcodes.UPPERTRI.toString())) {
 			CPOperand target = getTargetOperand();
-			CPOperand lower = getBoolLiteral("lowertri");
+			CPOperand lower = getBoolLiteral(Opcodes.LOWERTRI.toString());
 			CPOperand diag = getBoolLiteral("diag");
 			CPOperand values = getBoolLiteral("values");
 			return Pair.of(output.getName(),
 				new LineageItem(getOpcode(), LineageItemUtils.getLineage(ec, target, lower, diag, values)));
 		}
-		else if(opcode.equalsIgnoreCase("transformdecode") || opcode.equalsIgnoreCase("transformapply")) {
+		else if(opcode.equalsIgnoreCase(Opcodes.TRANSFORMDECODE.toString()) || opcode.equalsIgnoreCase(Opcodes.TRANSFORMAPPLY.toString())) {
 			CPOperand target = new CPOperand(params.get("target"), ValueType.FP64, DataType.FRAME);
 			CPOperand meta = getLiteral("meta", ValueType.UNKNOWN, DataType.FRAME);
-			CPOperand spec = getStringLiteral("spec");
+			CPOperand spec = new CPOperand(params.get("spec"), ValueType.STRING, DataType.SCALAR);
 			return Pair.of(output.getName(),
 				new LineageItem(getOpcode(), LineageItemUtils.getLineage(ec, target, meta, spec)));
 		}
-		else if (opcode.equalsIgnoreCase("nvlist") || opcode.equalsIgnoreCase("autoDiff")) {
+		else if (opcode.equalsIgnoreCase(Opcodes.NVLIST.toString()) || opcode.equalsIgnoreCase(Opcodes.AUTODIFF.toString())) {
 			List<String> names = new ArrayList<>(params.keySet());
 			CPOperand[] listOperands = names.stream().map(n -> ec.containsVariable(params.get(n)) 
 					? new CPOperand(n, ec.getVariable(params.get(n))) 
