@@ -42,6 +42,7 @@ import org.apache.spark.api.java.function.Function;
 import org.apache.spark.api.java.function.PairFunction;
 import org.apache.spark.util.random.SamplingUtils;
 import org.apache.sysds.api.DMLScript;
+import org.apache.sysds.common.Opcodes;
 import org.apache.sysds.common.Types.DataType;
 import org.apache.sysds.common.Types.ExecMode;
 import org.apache.sysds.common.Types.OpOpDG;
@@ -49,22 +50,20 @@ import org.apache.sysds.common.Types.ValueType;
 import org.apache.sysds.conf.ConfigurationManager;
 import org.apache.sysds.hops.DataGenOp;
 import org.apache.sysds.hops.OptimizerUtils;
-import org.apache.sysds.lops.DataGen;
 import org.apache.sysds.lops.Lop;
 import org.apache.sysds.parser.DataExpression;
 import org.apache.sysds.runtime.DMLRuntimeException;
 import org.apache.sysds.runtime.controlprogram.context.ExecutionContext;
 import org.apache.sysds.runtime.controlprogram.context.SparkExecutionContext;
-import org.apache.sysds.runtime.controlprogram.parfor.stat.InfrastructureAnalyzer;
 import org.apache.sysds.runtime.data.BasicTensorBlock;
 import org.apache.sysds.runtime.data.TensorBlock;
 import org.apache.sysds.runtime.data.TensorIndexes;
+import org.apache.sysds.runtime.frame.data.FrameBlock;
 import org.apache.sysds.runtime.instructions.InstructionUtils;
 import org.apache.sysds.runtime.instructions.cp.CPOperand;
 import org.apache.sysds.runtime.instructions.spark.utils.RDDConverterUtils;
 import org.apache.sysds.runtime.io.IOUtilFunctions;
 import org.apache.sysds.runtime.lineage.LineageItem;
-import org.apache.sysds.runtime.matrix.data.FrameBlock;
 import org.apache.sysds.runtime.matrix.data.LibMatrixDatagen;
 import org.apache.sysds.runtime.matrix.data.MatrixBlock;
 import org.apache.sysds.runtime.matrix.data.MatrixCell;
@@ -77,6 +76,7 @@ import org.apache.sysds.runtime.meta.TensorCharacteristics;
 import org.apache.sysds.runtime.util.DataConverter;
 import org.apache.sysds.runtime.util.UtilFunctions;
 import org.apache.sysds.utils.Statistics;
+import org.apache.sysds.utils.stats.InfrastructureAnalyzer;
 
 import scala.Array;
 import scala.Tuple2;
@@ -84,7 +84,7 @@ import scala.Tuple2;
 public class RandSPInstruction extends UnarySPInstruction {
 	private static final Log LOG = LogFactory.getLog(RandSPInstruction.class.getName());
 	// internal configuration
-	private static final long INMEMORY_NUMBLOCKS_THRESHOLD = 1024 * 1024;
+	public static final long INMEMORY_NUMBLOCKS_THRESHOLD = 1024 * 1024;
 
 	private OpOpDG _method = null;
 	private final CPOperand rows, cols, dims;
@@ -225,21 +225,21 @@ public class RandSPInstruction extends UnarySPInstruction {
 		String opcode = s[0];
 		
 		OpOpDG method = null;
-		if ( opcode.equalsIgnoreCase(DataGen.RAND_OPCODE) ) {
+		if ( opcode.equalsIgnoreCase(Opcodes.RANDOM.toString()) ) {
 			method = OpOpDG.RAND;
 			InstructionUtils.checkNumFields ( str, 10, 11 );
 		}
-		else if ( opcode.equalsIgnoreCase(DataGen.SEQ_OPCODE) ) {
+		else if ( opcode.equalsIgnoreCase(Opcodes.SEQUENCE.toString()) ) {
 			method = OpOpDG.SEQ;
 			// 8 operands: rows, cols, blen, from, to, incr, outvar
 			InstructionUtils.checkNumFields ( str, 7 );
 		}
-		else if ( opcode.equalsIgnoreCase(DataGen.SAMPLE_OPCODE) ) {
+		else if ( opcode.equalsIgnoreCase(Opcodes.SAMPLE.toString()) ) {
 			method = OpOpDG.SAMPLE;
 			// 7 operands: range, size, replace, seed, blen, outvar
 			InstructionUtils.checkNumFields ( str, 6 );
 		}
-		else if ( opcode.equalsIgnoreCase(DataGen.FRAME_OPCODE) ) {
+		else if ( opcode.equalsIgnoreCase("frame") ) {
 			method = OpOpDG.FRAMEINIT;
 			InstructionUtils.checkNumFields ( str, 6 );
 		}
@@ -1195,22 +1195,40 @@ public class RandSPInstruction extends UnarySPInstruction {
 	@Override
 	public Pair<String, LineageItem> getLineageItem(ExecutionContext ec) {
 		String tmpInstStr = instString;
-		if (getSeed() == DataGenOp.UNSPECIFIED_SEED) {
-			//generate pseudo-random seed (because not specified)
-			if (runtimeSeed == null)
-				runtimeSeed = (minValue == maxValue && sparsity == 1) ?
-					DataGenOp.UNSPECIFIED_SEED : DataGenOp.generateRandomSeed();
-			int position = (_method == OpOpDG.RAND) ? SEED_POSITION_RAND :
-				(_method == OpOpDG.SAMPLE) ? SEED_POSITION_SAMPLE : 0;
-			tmpInstStr = InstructionUtils.replaceOperand(
-				tmpInstStr, position, String.valueOf(runtimeSeed));
-			if( !rows.isLiteral() )
-				tmpInstStr = InstructionUtils.replaceOperand(tmpInstStr, 2,
-					new CPOperand(ec.getScalarInput(rows)).getLineageLiteral());
-			if( !cols.isLiteral() )
-				tmpInstStr = InstructionUtils.replaceOperand(tmpInstStr, 3,
-					new CPOperand(ec.getScalarInput(cols)).getLineageLiteral());
+		switch(_method) {
+			case RAND: {
+				tmpInstStr = InstructionUtils.replaceOperandName(tmpInstStr);
+				if(getSeed() == DataGenOp.UNSPECIFIED_SEED) {
+					//generate pseudo-random seed (because not specified)
+					if(runtimeSeed == null)
+						runtimeSeed = (minValue == maxValue && sparsity == 1) ? DataGenOp.UNSPECIFIED_SEED : DataGenOp.generateRandomSeed();
+					int position = (_method == OpOpDG.RAND) ? SEED_POSITION_RAND : (_method == OpOpDG.SAMPLE) ? SEED_POSITION_SAMPLE : 0;
+					tmpInstStr = InstructionUtils.replaceOperand(tmpInstStr, position, String.valueOf(runtimeSeed));
+					if(!rows.isLiteral())
+						tmpInstStr = InstructionUtils.replaceOperand(tmpInstStr, 2, new CPOperand(ec.getScalarInput(rows)).getLineageLiteral());
+					if(!cols.isLiteral())
+						tmpInstStr = InstructionUtils.replaceOperand(tmpInstStr, 3, new CPOperand(ec.getScalarInput(cols)).getLineageLiteral());
+				}
+				break;
+			}
+			case SEQ: {
+				tmpInstStr = InstructionUtils.replaceOperandName(tmpInstStr);
+				CPOperand blkSize = new CPOperand(String.valueOf(blocksize), ValueType.INT64, DataType.SCALAR, true);
+				tmpInstStr = InstructionUtils.replaceOperand(tmpInstStr, 4, blkSize.getLineageLiteral());
+				tmpInstStr = replaceNonLiteral(tmpInstStr, seq_from, 5, ec);
+				tmpInstStr = replaceNonLiteral(tmpInstStr, seq_to, 6, ec);
+				tmpInstStr = replaceNonLiteral(tmpInstStr, seq_incr, 7, ec);
+				break;
+			}
+			default:
+				throw new DMLRuntimeException("Unsupported Spark datagen op: " + _method);
 		}
 		return Pair.of(output.getName(), new LineageItem(tmpInstStr, getOpcode()));
 	}
+
+	private static String replaceNonLiteral(String inst, CPOperand op, int pos, ExecutionContext ec) {
+		if(!op.isLiteral())
+			inst = InstructionUtils.replaceOperand(inst, pos, new CPOperand(ec.getScalarInput(op)).getLineageLiteral());
+		return inst;
+	}
 }
