@@ -20,7 +20,6 @@ package org.apache.sysds.runtime.matrix.data;
 
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.List;
 import java.util.concurrent.Callable;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Future;
@@ -651,6 +650,37 @@ public class LibMatrixDNN {
 			params.output.setNonZeros(0);
 		}
 	}
+
+	public static void lstm(DnnParameters params){
+		long nnz;
+		if(LibMatrixDNNLSTM.checkLSTMInputForOptimisation(params)){
+			params.output.allocateDenseBlock();
+			params.output2.allocateDenseBlock();
+			params.output3.allocateDenseBlock();
+			params.output4.allocateDenseBlock();
+			params.output5.allocateDenseBlock();
+			nnz = execute(LibMatrixDNNLSTM.getLSTMWorkers(params), params);
+		}
+		else
+			nnz = LibMatrixDNNLSTM.lstmGeneric(params);
+		//post-processing: maintain nnz
+		params.output.setNonZeros(nnz);
+		params.output.examSparsity();
+	}
+
+	public static void lstmBackward(DnnParameters params) {
+		long nnz;
+		if(LibMatrixDNNLSTM.checkLSTMBackwardInputForOptimisation(params)){
+			//out.allocateDenseBlock();
+			//cout.allocateDenseBlock();
+			nnz = execute(LibMatrixDNNLSTM.getLSTMWorkers(params), params);
+		}
+		else
+			nnz = LibMatrixDNNLSTM.lstmBackwardGeneric(params);
+		//post-processing: maintain nnz
+		params.output.setNonZeros(nnz);
+		params.output.examSparsity();
+	}
 	
 	/**
 	 * Executes the tasks in parallel using java's ExecutorService.
@@ -671,10 +701,13 @@ public class LibMatrixDNN {
 			}
 			else {
 				ExecutorService pool = CommonThreadPool.get(k);
-				List<Future<Long>> taskret = pool.invokeAll(tasks);
-				pool.shutdown();
-				for( Future<Long> task : taskret )
-					lnnz += task.get();
+				try{
+					for( Future<Long> task : pool.invokeAll(tasks) )
+						lnnz += task.get();
+				}
+				finally{
+					pool.shutdown();
+				}
 			}
 		} 
 		catch (Exception e) {
