@@ -27,7 +27,8 @@ import java.util.Arrays;
 import java.util.List;
 
 import org.apache.sysds.common.Types.ValueType;
-import org.apache.sysds.runtime.matrix.data.FrameBlock;
+import org.apache.sysds.runtime.frame.data.FrameBlock;
+import org.apache.sysds.runtime.frame.data.columns.ColumnMetadata;
 import org.apache.sysds.runtime.matrix.data.MatrixBlock;
 import org.apache.sysds.runtime.util.UtilFunctions;
 
@@ -53,19 +54,24 @@ public class DecoderPassThrough extends Decoder
 	@Override
 	public FrameBlock decode(MatrixBlock in, FrameBlock out) {
 		out.ensureAllocatedColumns(in.getNumRows());
+		decode(in, out, 0, in.getNumRows());
+		return out;
+	}
+	
+	@Override
+	public void decode(MatrixBlock in, FrameBlock out, int rl, int ru) {
 		int clen = Math.min(_colList.length, out.getNumColumns());
-		for( int i=0; i<in.getNumRows(); i++ ) {
+		for( int i=rl; i<ru; i++ ) {
 			for( int j=0; j<clen; j++ ) {
 				int srcColID = _srcCols[j];
 				int tgtColID = _colList[j];
-				double val = in.quickGetValue(i, srcColID-1);
+				double val = in.get(i, srcColID-1);
 				out.set(i, tgtColID-1,
 					UtilFunctions.doubleToObject(_schema[tgtColID-1], val));
 			}
 		}
-		return out;
 	}
-	
+
 	@Override
 	public Decoder subRangeDecoder(int colStart, int colEnd, int dummycodedOffset) {
 		List<Integer> colList = new ArrayList<>();
@@ -107,7 +113,8 @@ public class DecoderPassThrough extends Decoder
 					ix1 ++;
 				}
 				else { //_colList[ix1] > _dcCols[ix2]
-					off += (int)meta.getColumnMetadata()[_dcCols[ix2]-1].getNumDistinct() - 1;
+					ColumnMetadata d =meta.getColumnMetadata()[_dcCols[ix2]-1];
+					off += d.isDefault() ? -1 : d.getNumDistinct() - 1;
 					ix2 ++;
 				}
 			}
