@@ -28,7 +28,6 @@ import java.util.Arrays;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Map;
-import java.util.Map.Entry;
 import java.util.Objects;
 import java.util.concurrent.Callable;
 
@@ -47,7 +46,7 @@ public class ColumnEncoderRecode extends ColumnEncoder {
 	public static boolean SORT_RECODE_MAP = false;
 
 	// recode maps and custom map for partial recode maps
-	private Map<Object, Long> _rcdMap;
+	private Map<Object, Integer> _rcdMap;
 	private HashSet<Object> _rcdMapPart = null;
 
 	public ColumnEncoderRecode(int colID) {
@@ -59,7 +58,7 @@ public class ColumnEncoderRecode extends ColumnEncoder {
 		this(-1);
 	}
 
-	protected ColumnEncoderRecode(int colID, HashMap<Object, Long> rcdMap) {
+	protected ColumnEncoderRecode(int colID, Map<Object, Integer> rcdMap) {
 		super(colID);
 		_rcdMap = rcdMap;
 	}
@@ -71,12 +70,12 @@ public class ColumnEncoderRecode extends ColumnEncoder {
 	 * @param code  is code for token
 	 * @return the concatenation of token and code with delimiter in between
 	 */
-	public static String constructRecodeMapEntry(String token, Long code) {
+	public static String constructRecodeMapEntry(String token, Integer code) {
 		StringBuilder sb = new StringBuilder(token.length() + 16);
 		return constructRecodeMapEntry(token, code, sb);
 	}
 
-	private static String constructRecodeMapEntry(Object token, Long code, StringBuilder sb) {
+	public static String constructRecodeMapEntry(Object token, Integer code, StringBuilder sb) {
 		sb.setLength(0); // reset reused string builder
 		return sb.append(token).append(Lop.DATATYPE_PREFIX).append(code.longValue()).toString();
 	}
@@ -94,7 +93,7 @@ public class ColumnEncoderRecode extends ColumnEncoder {
 		return new String[] {value.substring(0, pos), value.substring(pos + 1)};
 	}
 
-	public Map<Object, Long> getCPRecodeMaps() {
+	public Map<Object, Integer> getCPRecodeMaps() {
 		return _rcdMap;
 	}
 
@@ -106,7 +105,7 @@ public class ColumnEncoderRecode extends ColumnEncoder {
 		sortCPRecodeMaps(_rcdMap);
 	}
 
-	private static void sortCPRecodeMaps(Map<Object, Long> map) {
+	private static void sortCPRecodeMaps(Map<Object, Integer> map) {
 		Object[] keys = map.keySet().toArray(new Object[0]);
 		Arrays.sort(keys);
 		map.clear();
@@ -114,7 +113,7 @@ public class ColumnEncoderRecode extends ColumnEncoder {
 			putCode(map, key);
 	}
 
-	private static void makeRcdMap(CacheBlock<?> in, Map<Object, Long> map, int colID, int startRow, int blk) {
+	private static void makeRcdMap(CacheBlock<?> in, Map<Object, Integer> map, int colID, int startRow, int blk) {
 		for(int row = startRow; row < getEndIndex(in.getNumRows(), startRow, blk); row++){
 			String key = in.getString(row, colID - 1);
 			if(key != null && !key.isEmpty() && !map.containsKey(key))
@@ -126,10 +125,10 @@ public class ColumnEncoderRecode extends ColumnEncoder {
 	}
 
 	private long lookupRCDMap(Object key) {
-		return _rcdMap.getOrDefault(key, -1L);
+		return _rcdMap.getOrDefault(key, -1);
 	}
 
-	public void computeRCDMapSizeEstimate(CacheBlock<?> in, int[] sampleIndices) {
+	public void computeMapSizeEstimate(CacheBlock<?> in, int[] sampleIndices) {
 		if (getEstMetaSize() != 0)
 			return;
 
@@ -160,7 +159,8 @@ public class ColumnEncoderRecode extends ColumnEncoder {
 		// We assume each partial map contains all distinct values and have the same size
 		long avgKeySize = totSize / distinctFreq.size();
 		long valSize = 16L; //sizeof(Long) = 8 + header
-		long estMapSize = estDistCount * (avgKeySize + valSize);
+		this._avgEntrySize = avgKeySize + valSize;
+		long estMapSize = estDistCount * _avgEntrySize;
 		setEstMetaSize(estMapSize);
 	}
 
@@ -187,7 +187,7 @@ public class ColumnEncoderRecode extends ColumnEncoder {
 
 	@Override
 	public Callable<Object> getPartialBuildTask(CacheBlock<?> in, int startRow, 
-			int blockSize, HashMap<Integer, Object> ret) {
+			int blockSize, HashMap<Integer, Object> ret, int p) {
 		return new RecodePartialBuildTask(in, _colID, startRow, blockSize, ret);
 	}
 
@@ -202,8 +202,8 @@ public class ColumnEncoderRecode extends ColumnEncoder {
 	 * @param map column map
 	 * @param key key for the new entry
 	 */
-	protected static void putCode(Map<Object, Long> map, Object key) {
-		map.put(key, (long) (map.size() + 1));
+	protected static void putCode(Map<Object, Integer> map, Object key) {
+		map.put(key, (map.size() + 1));
 	}
 
 	protected double getCode(CacheBlock<?> in, int r){
@@ -269,10 +269,10 @@ public class ColumnEncoderRecode extends ColumnEncoder {
 		assert other._colID == _colID;
 		// merge together overlapping columns
 		ColumnEncoderRecode otherRec = (ColumnEncoderRecode) other;
-		Map<Object, Long> otherMap = otherRec._rcdMap;
+		Map<Object, Integer> otherMap = otherRec._rcdMap;
 		if(otherMap != null) {
 			// for each column, add all non present recode values
-			for(Map.Entry<Object, Long> entry : otherMap.entrySet()) {
+			for(Map.Entry<Object, Integer> entry : otherMap.entrySet()) {
 				if(lookupRCDMap(entry.getKey()) == -1) {
 					// key does not yet exist
 					putCode(_rcdMap, entry.getKey());
@@ -303,16 +303,27 @@ public class ColumnEncoderRecode extends ColumnEncoder {
 
 		// create compact meta data representation
 		StringBuilder sb = new StringBuilder(); // for reuse
-		int rowID = 0;
-		for(Entry<Object, Long> e : _rcdMap.entrySet()) {
-			meta.set(rowID++, _colID - 1, // 1-based
-				constructRecodeMapEntry(e.getKey(), e.getValue(), sb));
-		}
-		meta.getColumnMetadata(_colID - 1).setNumDistinct(getNumDistinctValues());
+		final Inc rowID = new Inc();
+	
+		final int colIDCorrected = _colID - 1;
+		_rcdMap.forEach( (k,v) -> {
+			meta.set(rowID.i(), colIDCorrected, // 1-based
+				constructRecodeMapEntry(k, v, sb));
+		});
+		// for(Entry<Object, Integer> e : _rcdMap.entrySet()) {
+		// }
+		meta.getColumnMetadata(colIDCorrected).setNumDistinct(getNumDistinctValues());
 
 		return meta;
 	}
 
+	private static class Inc{
+		int i = 0; 
+		public int i(){
+			return i++;
+		}
+	}
+
 	/**
 	 * Construct the recodemaps from the given input frame for all columns registered for recode.
 	 *
@@ -330,10 +341,15 @@ public class ColumnEncoderRecode extends ColumnEncoder {
 		super.writeExternal(out);
 		out.writeInt(_rcdMap.size());
 		
-		for(Entry<Object, Long> e : _rcdMap.entrySet()) {
-			out.writeUTF(e.getKey().toString());
-			out.writeLong(e.getValue());
-		}
+		_rcdMap.forEach((k, v)-> {
+			try{
+				out.writeUTF(k.toString());
+				out.writeInt(v);
+			}
+			catch(Exception e){
+				throw new RuntimeException(e);
+			}
+		});
 	}
 
 	@Override
@@ -342,7 +358,7 @@ public class ColumnEncoderRecode extends ColumnEncoder {
 		int size = in.readInt();
 		for(int j = 0; j < size; j++) {
 			String key = in.readUTF();
-			Long value = in.readLong();
+			Integer value = in.readInt();
 			_rcdMap.put(key, value);
 		}
 	}
@@ -362,7 +378,7 @@ public class ColumnEncoderRecode extends ColumnEncoder {
 		return Objects.hash(_rcdMap);
 	}
 
-	public Map<Object, Long> getRcdMap() {
+	public Map<Object, Integer> getRcdMap() {
 		return _rcdMap;
 	}
 
@@ -373,16 +389,16 @@ public class ColumnEncoderRecode extends ColumnEncoder {
 		sb.append(": ");
 		sb.append(_colID);
 		sb.append(" --- map: ");
-		sb.append(_rcdMap);
+		if(_rcdMap.size() < 1000){
+			sb.append(_rcdMap);
+		}
+		else{
+			sb.append("Map to big to print but size is : " + _rcdMap.size());
+		}
 		return sb.toString();
 	}
 
 	private static class RecodeSparseApplyTask extends ColumnApplyTask<ColumnEncoderRecode>{
-
-		public RecodeSparseApplyTask(ColumnEncoderRecode encoder, CacheBlock<?> input, MatrixBlock out, int outputCol) {
-			super(encoder, input, out, outputCol);
-		}
-
 		protected RecodeSparseApplyTask(ColumnEncoderRecode encoder, CacheBlock<?> input, MatrixBlock out,
 										int outputCol, int startRow, int blk) {
 			super(encoder, input, out, outputCol, startRow, blk);
@@ -429,7 +445,7 @@ public class ColumnEncoderRecode extends ColumnEncoder {
 		@Override
 		public Object call() throws Exception {
 			long t0 = DMLScript.STATISTICS ? System.nanoTime() : 0;
-			HashMap<Object, Long> partialMap = new HashMap<>();
+			HashMap<Object, Integer> partialMap = new HashMap<>();
 			makeRcdMap(_input, partialMap, _colID, _startRow, _blockSize);
 			synchronized(_partialMaps) {
 				_partialMaps.put(_startRow, partialMap);
@@ -459,7 +475,7 @@ public class ColumnEncoderRecode extends ColumnEncoder {
 		@Override
 		public Object call() throws Exception {
 			long t0 = DMLScript.STATISTICS ? System.nanoTime() : 0;
-			Map<Object, Long> rcdMap = _encoder.getRcdMap();
+			Map<Object, Integer> rcdMap = _encoder.getRcdMap();
 			_partialMaps.forEach((start_row, map) -> {
 				((HashMap<?, ?>) map).forEach((k, v) -> {
 					if(!rcdMap.containsKey(k))
