@@ -25,6 +25,7 @@ import java.util.Collection;
 import java.util.concurrent.atomic.AtomicBoolean;
 
 import org.apache.commons.lang3.NotImplementedException;
+import org.apache.sysds.common.Opcodes;
 import org.apache.sysds.runtime.compress.CompressedMatrixBlock;
 import org.apache.sysds.runtime.compress.CompressionSettingsBuilder;
 import org.apache.sysds.runtime.compress.colgroup.AColGroup.CompressionType;
@@ -216,37 +217,37 @@ public abstract class AbstractCompressedUnaryTests extends CompressedTestBase {
 	protected AggregateUnaryOperator getUnaryOperator(AggType aggType, int threads) {
 		switch(aggType) {
 			case SUM:
-				return InstructionUtils.parseBasicAggregateUnaryOperator("uak+", threads);
+				return InstructionUtils.parseBasicAggregateUnaryOperator(Opcodes.UAKP.toString(), threads);
 			case ROW_SUMS:
-				return InstructionUtils.parseBasicAggregateUnaryOperator("uark+", threads);
+				return InstructionUtils.parseBasicAggregateUnaryOperator(Opcodes.UARKP.toString(), threads);
 			case COL_SUMS:
-				return InstructionUtils.parseBasicAggregateUnaryOperator("uack+", threads);
+				return InstructionUtils.parseBasicAggregateUnaryOperator(Opcodes.UACKP.toString(), threads);
 			case SUM_SQ:
-				return InstructionUtils.parseBasicAggregateUnaryOperator("uasqk+", threads);
+				return InstructionUtils.parseBasicAggregateUnaryOperator(Opcodes.UASQKP.toString(), threads);
 			case ROW_SUMS_SQ:
-				return InstructionUtils.parseBasicAggregateUnaryOperator("uarsqk+", threads);
+				return InstructionUtils.parseBasicAggregateUnaryOperator(Opcodes.UARSQKP.toString(), threads);
 			case COL_SUMS_SQ:
-				return InstructionUtils.parseBasicAggregateUnaryOperator("uacsqk+", threads);
+				return InstructionUtils.parseBasicAggregateUnaryOperator(Opcodes.UACSQKP.toString(), threads);
 			case MAX:
-				return InstructionUtils.parseBasicAggregateUnaryOperator("uamax", threads);
+				return InstructionUtils.parseBasicAggregateUnaryOperator(Opcodes.UAMAX.toString(), threads);
 			case ROW_MAXS:
-				return InstructionUtils.parseBasicAggregateUnaryOperator("uarmax", threads);
+				return InstructionUtils.parseBasicAggregateUnaryOperator(Opcodes.UARMAX.toString(), threads);
 			case COL_MAXS:
-				return InstructionUtils.parseBasicAggregateUnaryOperator("uacmax", threads);
+				return InstructionUtils.parseBasicAggregateUnaryOperator(Opcodes.UACMAX.toString(), threads);
 			case MIN:
-				return InstructionUtils.parseBasicAggregateUnaryOperator("uamin", threads);
+				return InstructionUtils.parseBasicAggregateUnaryOperator(Opcodes.UAMIN.toString(), threads);
 			case ROW_MINS:
-				return InstructionUtils.parseBasicAggregateUnaryOperator("uarmin", threads);
+				return InstructionUtils.parseBasicAggregateUnaryOperator(Opcodes.UARMIN.toString(), threads);
 			case COL_MINS:
-				return InstructionUtils.parseBasicAggregateUnaryOperator("uacmin", threads);
+				return InstructionUtils.parseBasicAggregateUnaryOperator(Opcodes.UACMIN.toString(), threads);
 			case MEAN:
-				return InstructionUtils.parseBasicAggregateUnaryOperator("uamean", threads);
+				return InstructionUtils.parseBasicAggregateUnaryOperator(Opcodes.UAMEAN.toString(), threads);
 			case ROW_MEAN:
-				return InstructionUtils.parseBasicAggregateUnaryOperator("uarmean", threads);
+				return InstructionUtils.parseBasicAggregateUnaryOperator(Opcodes.UARMEAN.toString(), threads);
 			case COL_MEAN:
-				return InstructionUtils.parseBasicAggregateUnaryOperator("uacmean", threads);
+				return InstructionUtils.parseBasicAggregateUnaryOperator(Opcodes.UACMEAN.toString(), threads);
 			case PRODUCT:
-				return InstructionUtils.parseBasicAggregateUnaryOperator("ua*", threads);
+				return InstructionUtils.parseBasicAggregateUnaryOperator(Opcodes.UAM.toString(), threads);
 			default:
 				throw new NotImplementedException("Not Supported Aggregate Unary operator in test");
 		}
@@ -299,20 +300,20 @@ public abstract class AbstractCompressedUnaryTests extends CompressedTestBase {
 			}
 			else {
 				if(aggType == AggType.PRODUCT) {
-					if(Double.isInfinite(ret2.quickGetValue(0, 0))){
+					if(Double.isInfinite(ret2.get(0, 0))){
 						if(!printedWarningForProduct.get()) {
 							printedWarningForProduct.set(true);
 							LOG.warn("Product not equal because of double rounding upwards");
 						}
-						return;	
+						return;
 					}
-					if(Math.abs(ret2.quickGetValue(0, 0)) <= 1E-16) {
+					if(Math.abs(ret2.get(0, 0)) <= 1E-16) {
 						if(!printedWarningForProduct.get()) {
 							printedWarningForProduct.set(true);
 							LOG.warn("Product not equal because of  double rounding downwards");
 						}
 
-						return; // Product is wierd around zero.
+						return; // Product is weird around zero.
 					}
 					TestUtils.compareMatricesPercentageDistance(ret1, ret2, 0.95, 0.98, css);
 				}
