@@ -19,6 +19,7 @@
 
 package org.apache.sysds.test.component.compress;
 
+import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
 import java.util.ArrayList;
@@ -26,10 +27,9 @@ import java.util.Arrays;
 import java.util.Collection;
 import java.util.EnumSet;
 import java.util.List;
-import java.util.concurrent.ExecutionException;
 import java.util.concurrent.atomic.AtomicBoolean;
 
-import org.apache.commons.lang.NotImplementedException;
+import org.apache.commons.lang3.NotImplementedException;
 import org.apache.commons.lang3.tuple.Pair;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
@@ -48,15 +48,18 @@ import org.apache.sysds.runtime.compress.colgroup.AColGroup;
 import org.apache.sysds.runtime.compress.colgroup.AColGroup.CompressionType;
 import org.apache.sysds.runtime.compress.colgroup.ColGroupFactory;
 import org.apache.sysds.runtime.compress.colgroup.ColGroupUncompressed;
+import org.apache.sysds.runtime.compress.colgroup.indexes.ColIndexFactory;
+import org.apache.sysds.runtime.compress.colgroup.indexes.IColIndex;
+import org.apache.sysds.runtime.compress.colgroup.indexes.IIterate;
 import org.apache.sysds.runtime.compress.cost.ACostEstimate;
 import org.apache.sysds.runtime.compress.cost.CostEstimatorBuilder;
 import org.apache.sysds.runtime.compress.cost.CostEstimatorFactory;
 import org.apache.sysds.runtime.compress.cost.CostEstimatorFactory.CostType;
 import org.apache.sysds.runtime.compress.cost.InstructionTypeCounter;
-import org.apache.sysds.runtime.compress.estim.CompressedSizeEstimatorFactory;
+import org.apache.sysds.runtime.compress.estim.ComEstFactory;
 import org.apache.sysds.runtime.compress.estim.CompressedSizeInfo;
 import org.apache.sysds.runtime.compress.estim.CompressedSizeInfoColGroup;
-import org.apache.sysds.runtime.compress.lib.CLALibAppend;
+import org.apache.sysds.runtime.compress.lib.CLALibCBind;
 import org.apache.sysds.runtime.functionobjects.Builtin;
 import org.apache.sysds.runtime.functionobjects.Builtin.BuiltinCode;
 import org.apache.sysds.runtime.functionobjects.Divide;
@@ -90,29 +93,48 @@ import org.junit.runners.Parameterized.Parameters;
 public abstract class CompressedTestBase extends TestBase {
 	protected static final Log LOG = LogFactory.getLog(CompressedTestBase.class.getName());
 
-	protected static SparsityType[] usedSparsityTypes = new SparsityType[] {SparsityType.FULL, SparsityType.SPARSE,
-		SparsityType.ULTRA_SPARSE};
+	protected static SparsityType[] usedSparsityTypes = new SparsityType[] { //
+		SparsityType.FULL, //
+		// SparsityType.SPARSE, //
+		SparsityType.ULTRA_SPARSE //
+	};
 
-	protected static ValueType[] usedValueTypes = new ValueType[] {ValueType.RAND_ROUND, ValueType.OLE_COMPRESSIBLE,
-		ValueType.RLE_COMPRESSIBLE, ValueType.CONST};
+	protected static ValueType[] usedValueTypes = new ValueType[] { //
+		ValueType.RAND_ROUND, //
+		ValueType.OLE_COMPRESSIBLE, //
+		// ValueType.RLE_COMPRESSIBLE, //
+		ValueType.CONST //
+	};
 
-	protected static ValueRange[] usedValueRanges = new ValueRange[] {ValueRange.BOOLEAN, ValueRange.SMALL,
-		ValueRange.NEGATIVE};
+	protected static ValueRange[] usedValueRanges = new ValueRange[] { //
+		ValueRange.BOOLEAN, //
+		ValueRange.SMALL, //
+		// ValueRange.NEGATIVE, //
+		ValueRange.LARGE //
+	};
 
-	protected static OverLapping[] overLapping = new OverLapping[] {OverLapping.PLUS_LARGE, OverLapping.PLUS_ROW_VECTOR,
-		OverLapping.MATRIX, OverLapping.NONE, OverLapping.APPEND_CONST, OverLapping.C_BIND_SELF};
+	protected static OverLapping[] overLapping = new OverLapping[] { //
+		OverLapping.PLUS_LARGE, //
+		OverLapping.PLUS_ROW_VECTOR, //
+		OverLapping.MATRIX, //
+		OverLapping.NONE, //
+		OverLapping.APPEND_CONST, //
+		OverLapping.C_BIND_SELF //
+	};
 
-	protected static CompressionSettingsBuilder[] usedCompressionSettings = new CompressionSettingsBuilder[] {
-		// CLA TESTS!
+	protected static CompressionSettingsBuilder[] usedCompressionSettings = new CompressionSettingsBuilder[] { // s
 		// only DDC
 		// csb().setValidCompressions(EnumSet.of(CompressionType.DDC)),
 		// only SDC
 		// csb().setValidCompressions(EnumSet.of(CompressionType.SDC)),
 		// default settings
-		csb()};
+		csb() //
+	};
 
 	protected static MatrixTypology[] usedMatrixTypology = new MatrixTypology[] { // Selected Matrix Types
-		MatrixTypology.SMALL, MatrixTypology.LARGE};
+		MatrixTypology.SMALL, //
+		// MatrixTypology.LARGE //
+	};
 
 	private static final int compressionSeed = 7;
 	protected MatrixBlock cmb;
@@ -135,7 +157,7 @@ public abstract class CompressedTestBase extends TestBase {
 
 		_k = parallelism;
 		mb.examSparsity();
-		// CompressionSettings.PAR_DDC_THRESHOLD = 1;
+		CompressionSettings.PAR_DDC_THRESHOLD = 1;
 
 		try {
 			if(_cs == null && ct == null) {
@@ -167,8 +189,8 @@ public abstract class CompressedTestBase extends TestBase {
 						colIndexes[x] = y;
 					}
 
-					CompressedSizeInfoColGroup cgi = CompressedSizeEstimatorFactory.createEstimator(mb, cs, _k)
-						.getColGroupInfo(colIndexes);
+					CompressedSizeInfoColGroup cgi = ComEstFactory.createEstimator(mb, cs, _k)
+						.getColGroupInfo(ColIndexFactory.create(colIndexes));
 					CompressedSizeInfo csi = new CompressedSizeInfo(cgi);
 
 					ACostEstimate ce = CostEstimatorFactory.create(cs, ceb, mb.getNumRows(), mb.getNumColumns(),
@@ -193,9 +215,7 @@ public abstract class CompressedTestBase extends TestBase {
 					 * In case only Uncompressed and Const colgroups are available. filter the big tests from uncompressed
 					 * colgroup tests since the functionality should be verified even with smaller matrices
 					 */
-					int[] colIndexes = new int[mb.getNumColumns()];
-					for(int i = 0; i < colIndexes.length; i++)
-						colIndexes[i] = i;
+					IColIndex colIndexes = ColIndexFactory.create(mb.getNumColumns());
 					cmb = new CompressedMatrixBlock(mb.getNumRows(), mb.getNumColumns());
 					((CompressedMatrixBlock) cmb).allocateColGroup(ColGroupUncompressed.create(colIndexes, mb, false));
 
@@ -236,7 +256,7 @@ public abstract class CompressedTestBase extends TestBase {
 				case C_BIND_SELF:
 					if(cmb instanceof CompressedMatrixBlock) {
 						CompressedMatrixBlock cmbc = (CompressedMatrixBlock) cmb;
-						cmb = CLALibAppend.append(cmbc, cmbc, _k);
+						cmb = CLALibCBind.cbind(cmbc, cmbc, _k);
 						mb = mb.append(mb, new MatrixBlock());
 						cols *= 2;
 					}
@@ -248,7 +268,7 @@ public abstract class CompressedTestBase extends TestBase {
 				throw new DMLCompressionException("Matrix block gone");
 
 			if(cmb instanceof CompressedMatrixBlock) {
-				if(tmp != null && ov == OverLapping.APPEND_EMPTY || ov == OverLapping.APPEND_CONST) {
+				if(tmp != null && (ov == OverLapping.APPEND_EMPTY || ov == OverLapping.APPEND_CONST)) {
 					mb = mb.append(tmp, new MatrixBlock());
 					cmb = cmb.append(tmp, new MatrixBlock());
 					cols += tmp.getNumColumns();
@@ -287,7 +307,8 @@ public abstract class CompressedTestBase extends TestBase {
 					}
 					else if(ov == OverLapping.PLUS_ROW_VECTOR) {
 
-						MatrixBlock v = TestUtils.generateTestMatrixBlock(1, cols, -1, 1, 1.0, 4);
+						MatrixBlock v = TestUtils.generateTestMatrixBlock(1, cols, 0, 4, 1.0, 4);
+						v = TestUtils.ceil(v);
 						BinaryOperator bop = new BinaryOperator(Plus.getPlusFnObject(), _k);
 						mb = mb.binaryOperations(bop, v, null);
 						cmb = cmb.binaryOperations(bop, v, null);
@@ -317,7 +338,7 @@ public abstract class CompressedTestBase extends TestBase {
 	 * @param valueRange The value range used as input
 	 */
 	private void setLossyTolerance(ValueRange valueRange) {
-		lossyTolerance = (double) (Math.max(TestConstants.getMaxRangeValue(valueRange),
+		lossyTolerance = (Math.max(TestConstants.getMaxRangeValue(valueRange),
 			Math.abs(TestConstants.getMinRangeValue(valueRange)))) * (1.0 / 127.0) / 2.0;
 	}
 
@@ -389,7 +410,7 @@ public abstract class CompressedTestBase extends TestBase {
 			ov, null, null});
 
 		CompressionSettingsBuilder sb = csb().setCostType(CostType.W_TREE);
-		InstructionTypeCounter itc = new InstructionTypeCounter(10, 10, 0, 100, 10, 0, 0, 10, 50, false);
+		InstructionTypeCounter itc = new InstructionTypeCounter(10, 10, 0, 100, 10, 0, 0, 10, false);
 		CostEstimatorBuilder csb = new CostEstimatorBuilder(itc);
 		SparsityType st = SparsityType.THIRTY;
 		ValueType vt = ValueType.ONE_HOT;
@@ -429,7 +450,6 @@ public abstract class CompressedTestBase extends TestBase {
 				return; // Input was not compressed then just pass test
 			((CompressedMatrixBlock) cmb).clearSoftReferenceToDecompressed();
 			MatrixBlock decompressedMatrixBlock = ((CompressedMatrixBlock) cmb).decompress(_k);
-			// LOG.error(cmb);
 			compareResultMatrices(mb, decompressedMatrixBlock, 1);
 			if(mb.getNonZeros() != decompressedMatrixBlock.getNonZeros())
 				fail(bufferedToString + "\n NonZeros not equivalent: expected:" + mb.getNonZeros() + " was: "
@@ -485,13 +505,15 @@ public abstract class CompressedTestBase extends TestBase {
 
 	@Test
 	public void testVectorMatrixMult() {
-		MatrixBlock vector = TestUtils.generateTestMatrixBlock(1, rows, 0.9, 1.5, 1.0, 3);
+		MatrixBlock vector = TestUtils.generateTestMatrixBlock(1, rows, 0, 5, 1.0, 3);
+		vector = TestUtils.ceil(vector);
 		testLeftMatrixMatrix(vector);
 	}
 
 	@Test
 	public void testLeftMatrixMatrixMultSmall() {
-		MatrixBlock matrix = TestUtils.generateTestMatrixBlock(3, rows, 0.9, 1.5, 1.0, 3);
+		MatrixBlock matrix = TestUtils.generateTestMatrixBlock(3, rows, 0, 5, 1.0, 3);
+		matrix = TestUtils.ceil(matrix);
 		testLeftMatrixMatrix(matrix);
 	}
 
@@ -503,37 +525,38 @@ public abstract class CompressedTestBase extends TestBase {
 
 	@Test
 	public void testLeftMatrixMatrixMultSparse() {
-		MatrixBlock matrix = TestUtils.generateTestMatrixBlock(2, rows, 0.9, 1.5, .1, 3);
+		MatrixBlock matrix = TestUtils.generateTestMatrixBlock(2, rows, 0, 5, .1, 3);
+		matrix = TestUtils.ceil(matrix);
 		testLeftMatrixMatrix(matrix);
 	}
 
 	@Test
 	public void testLeftMatrixMatrixMultSparseCustom() {
 		MatrixBlock matrix = new MatrixBlock(2, rows, true);
-		matrix.quickSetValue(1, rows - 1, 99);
+		matrix.set(1, rows - 1, 99);
 		testLeftMatrixMatrix(matrix);
 	}
 
 	@Test
 	public void testLeftMatrixMatrixMultSparseCustom2() {
 		MatrixBlock matrix = new MatrixBlock(2, rows, true);
-		matrix.quickSetValue(1, 0, 99);
+		matrix.set(1, 0, 99);
 		testLeftMatrixMatrix(matrix);
 	}
 
 	@Test
 	public void testLeftMatrixMatrixMultSparseCustom3() {
 		MatrixBlock matrix = new MatrixBlock(2, rows, true);
-		matrix.quickSetValue(0, 0, -99);
-		matrix.quickSetValue(1, 0, 99);
+		matrix.set(0, 0, -99);
+		matrix.set(1, 0, 99);
 		testLeftMatrixMatrix(matrix);
 	}
 
 	@Test
 	public void testLeftMatrixMatrixMultSparseCustom4() {
 		MatrixBlock matrix = new MatrixBlock(2, rows, true);
-		matrix.quickSetValue(0, rows - 1, -99);
-		matrix.quickSetValue(1, 0, 99);
+		matrix.set(0, rows - 1, -99);
+		matrix.set(1, 0, 99);
 		testLeftMatrixMatrix(matrix);
 	}
 
@@ -596,7 +619,7 @@ public abstract class CompressedTestBase extends TestBase {
 
 	public void testRightMatrixMatrix(MatrixBlock matrix) {
 		try {
-			matrix.quickSetValue(0, 0, 10);
+			matrix.set(0, 0, 10);
 			// Make Operator
 			AggregateBinaryOperator abop = InstructionUtils.getMatMultOperator(_k);
 
@@ -666,6 +689,8 @@ public abstract class CompressedTestBase extends TestBase {
 			if(compressMatrix && !(compMatrix instanceof CompressedMatrixBlock))
 				return; // Early termination since the test does not test what we wanted.
 
+			compareResultMatrices(matrix, compMatrix, 0.0);
+
 			// Make Operator
 			AggregateBinaryOperator abopSingle = InstructionUtils.getMatMultOperator(1);
 
@@ -931,14 +956,14 @@ public abstract class CompressedTestBase extends TestBase {
 		}
 		catch(NotImplementedException e) {
 			if(!printedErrorForNotImplementedTestBinaryVMPlus.get()) {
-				LOG.error("Failed Binary VM Plus: " + e.getMessage());
+				LOG.error("Failed Left " + e.getMessage());
 				printedErrorForNotImplementedTestBinaryVMPlus.set(true);
 			}
 		}
 		catch(Exception e) {
-			if(e.getCause() instanceof ExecutionException && e.getCause().getCause() instanceof NotImplementedException) {
+			if(e.getCause() instanceof NotImplementedException) {
 				if(!printedErrorForNotImplementedTestBinaryVMPlus.get()) {
-					LOG.error("Failed Binary VM Plus: " + e.getMessage());
+					LOG.error("Failed Left " + e.getMessage());
 					printedErrorForNotImplementedTestBinaryVMPlus.set(true);
 				}
 			}
@@ -1032,10 +1057,103 @@ public abstract class CompressedTestBase extends TestBase {
 		}
 	}
 
+
+	@Test
+	public void testReshape2() {
+		testReshape(2);
+	}
+
+	@Test
+	public void testReshape3() {
+		testReshape(3);
+	}
+
+	@Test
+	public void testReshape10() {
+		testReshape(10);
+	}
+
+	/**
+	 * Test the reshape mechanic of the compressed block by reshaping the matrix by making it x times wider.
+	 * 
+	 * @param multiplier the multiplier x.
+	 */
+	public void testReshape(int multiplier) {
+		try {
+			if((double) rows / multiplier != rows / multiplier)
+				return;
+
+			final MatrixBlock ret2 = cmb.reshape(rows / multiplier, cols * multiplier, true);
+			final MatrixBlock ret1 = mb.reshape(rows / multiplier, cols * multiplier, true);
+			compareResultMatrices(ret1, ret2, 1);
+		}
+		catch(Exception e) {
+			e.printStackTrace();
+			throw new DMLRuntimeException("Error in Reshape", e);
+		}
+	}
+
+	@Test
+	public void testReshape2_divider() {
+		testReshapeDivider(2);
+	}
+
+	@Test
+	public void testReshape3_divider() {
+		testReshapeDivider(3);
+	}
+
+	@Test
+	public void testReshape10_divider() {
+		testReshapeDivider(10);
+	}
+
+	/**
+	 * Test the reshape mechanic of the compressed block by reshaping the matrix by making it x times taller.
+	 * 
+	 * @param divider the divider x.
+	 */
+	public void testReshapeDivider(int divider) {
+		try {
+			if((double) cols /divider != cols / divider)
+				return;
+
+			final MatrixBlock ret2 = cmb.reshape(rows * divider, cols / divider, true);
+			final MatrixBlock ret1 = mb.reshape(rows * divider, cols / divider, true);
+			compareResultMatrices(ret1, ret2, 1);
+		}
+		catch(Exception e) {
+			e.printStackTrace();
+			throw new DMLRuntimeException("Error in Reshape", e);
+		}
+	}
+
+
+	@Test
+	public void testReshape_opposite() {
+		testReshape(cols, rows);
+	}
+
+	public void testReshape(int newRows, int newCols) {
+		try {
+			if((double) newRows * newCols != rows * cols)
+				return;
+
+			final MatrixBlock ret2 = cmb.reshape(newRows, newCols, true);
+			final MatrixBlock ret1 = mb.reshape(newRows, newCols, true);
+			compareResultMatrices(ret1, ret2, 1);
+		}
+		catch(Exception e) {
+			e.printStackTrace();
+			throw new DMLRuntimeException("Error in Reshape", e);
+		}
+	}
+
 	@Test
 	public void testCompressAgain() {
 		try {
 			TestUtils.assertEqualColsAndRows(mb, cmb);
+			compareResultMatrices(mb, cmb, 1);
 			MatrixBlock cmba = CompressedMatrixBlockFactory.compress(cmb, _k).getLeft();
 			compareResultMatrices(mb, cmba, 1);
 		}
@@ -1057,6 +1175,45 @@ public abstract class CompressedTestBase extends TestBase {
 		compareResultMatrices(ret1, ret2, 1);
 	}
 
+	@Test
+	public void appendMultiple() {
+		try {
+
+			if(!(cmb instanceof CompressedMatrixBlock))
+				return;
+			if(vectorRows == null)
+				vectorRows = CompressibleInputGenerator.getInput(rows, 1, CompressionType.OLE, 5, 5, -5, 1.0, 3);
+			MatrixBlock ap = CompressedMatrixBlockFactory.compress(vectorRows).getLeft();
+			MatrixBlock ret1 = mb.append(new MatrixBlock[] {ap, ap}, new MatrixBlock(), true);
+			MatrixBlock ret2 = cmb.append(new MatrixBlock[] {ap, ap}, new MatrixBlock(), true);
+			compareResultMatrices(ret1, ret2, 1);
+		}
+		catch(Exception e) {
+			e.printStackTrace();
+			fail(e.getMessage());
+		}
+	}
+
+	@Test
+	public void appendMultipleNotAllCompressed() {
+		try {
+
+			if(!(cmb instanceof CompressedMatrixBlock))
+				return;
+			if(vectorRows == null)
+				vectorRows = CompressibleInputGenerator.getInput(rows, 1, CompressionType.OLE, 5, 5, -5, 1.0, 3);
+			MatrixBlock ap = vectorRows;
+			MatrixBlock apc = CompressedMatrixBlockFactory.compress(vectorRows).getLeft();
+			MatrixBlock ret1 = mb.append(new MatrixBlock[] {ap, apc}, new MatrixBlock(), true);
+			MatrixBlock ret2 = cmb.append(new MatrixBlock[] {ap, apc}, new MatrixBlock(), true);
+			compareResultMatrices(ret1, ret2, 1);
+		}
+		catch(Exception e) {
+			e.printStackTrace();
+			fail(e.getMessage());
+		}
+	}
+
 	@Test
 	public void appendCBindTrue() {
 		if(!(cmb instanceof CompressedMatrixBlock) || rows * cols > 10000)
@@ -1067,6 +1224,55 @@ public abstract class CompressedTestBase extends TestBase {
 		compareResultMatrices(ret1, ret2, 1);
 	}
 
+	@Test
+	public void appendCBindTrueEmpty() {
+		if(!(cmb instanceof CompressedMatrixBlock) || rows * cols > 10000)
+			return;
+		MatrixBlock ap = new MatrixBlock(mb.getNumRows(), 1, 0.0);
+		assertTrue(ap.isEmpty());
+		MatrixBlock ret1 = mb.append(ap, new MatrixBlock(), true);
+		MatrixBlock ret2 = cmb.append(ap, new MatrixBlock(), true);
+		compareResultMatrices(ret1, ret2, 1);
+	}
+
+	@Test
+	public void appendCBindTrueEmptyInverse() {
+		if(!(cmb instanceof CompressedMatrixBlock) || rows * cols > 10000)
+			return;
+		MatrixBlock ap = new MatrixBlock(mb.getNumRows(), 1, 0.0);
+		assertTrue(ap.isEmpty());
+		MatrixBlock ret1 = ap.append(mb, new MatrixBlock(), true);
+		MatrixBlock ret2 = ap.append(cmb, new MatrixBlock(), true);
+		compareResultMatrices(ret1, ret2, 1);
+	}
+
+	@Test
+	public void appendCBindTrueConstInverse() {
+		if(!(cmb instanceof CompressedMatrixBlock) || rows * cols > 10000)
+			return;
+		MatrixBlock ap = new MatrixBlock(mb.getNumRows(), 1, 2.0);
+		assertTrue(!ap.isEmpty());
+		MatrixBlock ret1 = ap.append(mb, new MatrixBlock(), true);
+		MatrixBlock ret2 = ap.append(cmb, new MatrixBlock(), true);
+		compareResultMatrices(ret1, ret2, 1);
+	}
+
+	@Test
+	public void appendCBindAlignedSelfMultiple() {
+		try {
+
+			if(!(cmb instanceof CompressedMatrixBlock) || rows * cols > 10000)
+				return;
+			MatrixBlock ret1 = mb.append(new MatrixBlock[] {mb, mb}, new MatrixBlock(), true);
+			MatrixBlock ret2 = cmb.append(new MatrixBlock[] {cmb, cmb}, new MatrixBlock(), true);
+			compareResultMatrices(ret1, ret2, 1);
+		}
+		catch(AssertionError e) {
+			e.printStackTrace();
+			fail("failed Cbind: " + cmb.toString());
+		}
+	}
+
 	@Test
 	public void appendCBindFalse() {
 		if(!(cmb instanceof CompressedMatrixBlock) || rows * cols > 10000)
@@ -1121,14 +1327,14 @@ public abstract class CompressedTestBase extends TestBase {
 
 	protected void compareResultMatrices(MatrixBlock expected, MatrixBlock result, double toleranceMultiplier) {
 		TestUtils.assertEqualColsAndRows(expected, result);
-		if(expected instanceof CompressedMatrixBlock)
+		if(expected instanceof CompressedMatrixBlock) {
+			verifyContainsAllColumns((CompressedMatrixBlock) expected);
 			expected = ((CompressedMatrixBlock) expected).decompress();
-		if(result instanceof CompressedMatrixBlock)
+		}
+		if(result instanceof CompressedMatrixBlock) {
+			verifyContainsAllColumns((CompressedMatrixBlock) result);
 			result = ((CompressedMatrixBlock) result).decompress();
-
-		if(result.getNonZeros() < expected.getNonZeros())
-			fail("Nonzero is to low guarantee at least equal or higher" + result.getNonZeros() + " vs "
-				+ expected.getNonZeros());
+		}
 
 		if(_cs != null && _cs.lossy)
 			TestUtils.compareMatricesPercentageDistance(expected, result, 0.25, 0.83, bufferedToString);
@@ -1141,15 +1347,45 @@ public abstract class CompressedTestBase extends TestBase {
 		else
 			TestUtils.compareMatricesBitAvgDistance(expected, result, (long) (27000 * toleranceMultiplier),
 				(long) (1024 * toleranceMultiplier), bufferedToString);
+
+		if(result.getNonZeros() != -1 && expected.getNonZeros() != -1 && result.getNonZeros() < expected.getNonZeros())
+			fail("Nonzero is to low guarantee at least equal or higher " + result.getNonZeros() + " vs "
+				+ expected.getNonZeros());
+
+	}
+
+	protected void verifyContainsAllColumns(CompressedMatrixBlock mb) {
+		boolean[] cols = new boolean[mb.getNumColumns()];
+		List<AColGroup> groups = mb.getColGroups();
+
+		for(int i = 0; i < groups.size(); i++) {
+			AColGroup g = groups.get(i);
+			IColIndex idx = g.getColIndices();
+			IIterate it = idx.iterator();
+			while(it.hasNext()) {
+				cols[it.v()] = true;
+				it.next();
+			}
+		}
+
+		for(int i = 0; i < cols.length; i++) {
+			if(!cols[i])
+				fail("Invalid constructed compression is missing column: " + i);
+		}
+
 	}
 
 	protected void compareResultMatricesPercentDistance(MatrixBlock expected, MatrixBlock result, double avg,
 		double max) {
 		TestUtils.assertEqualColsAndRows(expected, result);
-		if(expected instanceof CompressedMatrixBlock)
+		if(expected instanceof CompressedMatrixBlock) {
+			verifyContainsAllColumns((CompressedMatrixBlock) expected);
 			expected = ((CompressedMatrixBlock) expected).decompress();
-		if(result instanceof CompressedMatrixBlock)
+		}
+		if(result instanceof CompressedMatrixBlock) {
+			verifyContainsAllColumns((CompressedMatrixBlock) result);
 			result = ((CompressedMatrixBlock) result).decompress();
+		}
 
 		TestUtils.compareMatricesPercentageDistance(expected, result, avg, max, bufferedToString);
 
@@ -1159,4 +1395,42 @@ public abstract class CompressedTestBase extends TestBase {
 		return new CompressionSettingsBuilder().setSeed(compressionSeed).setMinimumSampleSize(100);
 	}
 
+	@Test
+	public void testReplaceNotContainedValue() {
+		double v = min - 1;
+		if(v != 0)
+			testReplace(v, 132);
+	}
+
+	@Test
+	public void testReplace() {
+		if(min != 0)
+			testReplace(min, 323);
+	}
+
+	@Test
+	public void testReplaceWithZero() {
+		if(min != 0)
+			testReplace(min, 0);
+	}
+
+	@Test
+	public void testReplaceZero() {
+		testReplace(0, 3232);
+	}
+
+	private void testReplace(double value, double replacements) {
+		try {
+			if(!(cmb instanceof CompressedMatrixBlock) || rows * cols > 10000)
+				return;
+			ucRet = mb.replaceOperations(ucRet, value, replacements, _k);
+			MatrixBlock ret2 = cmb.replaceOperations(new MatrixBlock(), value, replacements, _k);
+			compareResultMatrices(ucRet, ret2, 1);
+		}
+		catch(Exception e) {
+			e.printStackTrace();
+			throw new DMLRuntimeException(e);
+		}
+	}
+
 }
